package eu.etaxonomy.taxeditor.prototype2.view;

import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICellEditorListener;
import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.PlatformUI;

import com.swtdesigner.ResourceManager;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.Activator;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionAddQuickName;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionMoveTaxon;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionOpenNameEditor;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionOpenNewChildNameEditor;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionRemoveTaxonFromTree;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionSaveTaxon;
import eu.etaxonomy.taxeditor.prototype2.model.NameTreeContentProvider;

/**
 * Taxon tree viewer which responds to events within individual taxa.
 * 
 * Good overview of TreeViewer: 
 *   http://www.eclipse.org/articles/Article-TreeViewer/TreeViewerArticle.htm
 * 
 * @author p.ciardelli
 *
 */
public class TaxonomicTreeViewer extends TreeViewer {
	
	/**
	 * Creates a lazy-loading taxonomic tree that is sorted by TitleCache,
	 *  and which listens for name changes and new taxa
	 *  
	 * @param parent
	 */
	public TaxonomicTreeViewer(Composite parent) {
		
		// SW.VIRTUAL causes nodes to be loaded on-demand, improving performance
		super(parent, SWT.VIRTUAL);

		// Tree settings
		final Tree tree = this.getTree();
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true));
		tree.setLinesVisible(false);
		tree.setHeaderVisible(false);
		
		createContent();	
		createDoubleClickListener();
		createRightClickListener();
		createQuickAdd();
		createDragAndDrop();
		createToolTips();
	}
	
	/**
	 * Tool tips are used to show warnings; there is no tooltip mechanism for
	 *  individual tree items out-of-the-bo, so tipListener belows builds one
	 */
	private void createToolTips() {
		Tree tree = this.getTree();
		
		// Disable native tooltip
		tree.setToolTipText ("");
		
		tree.addListener (SWT.Dispose, tipListener);
		tree.addListener (SWT.KeyDown, tipListener);
		tree.addListener (SWT.MouseMove, tipListener);
		tree.addListener (SWT.MouseHover, tipListener);
	}

	/**
	 * Add drag and drop functionality
	 */
	private void createDragAndDrop() {
		final Tree tree = this.getTree();
		Transfer[] types = new Transfer[] {TextTransfer.getInstance()};
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_LINK;
		
		final DragSource source = new DragSource (tree, operations);
		source.setTransfer(types);
		final TreeItem[] dragSourceItem = new TreeItem[1];
		source.addDragListener (new DragSourceListener () {
			public void dragStart(DragSourceEvent event) {
				TreeItem[] selection = tree.getSelection();
				if (selection.length > 0) {
					event.doit = true;
					dragSourceItem[0] = selection[0];
				} else {
					event.doit = false;
				}
			}
			public void dragSetData (DragSourceEvent event) {
				event.data = dragSourceItem[0].getText();
			}
			public void dragFinished(DragSourceEvent event) {
				if (event.detail == DND.DROP_MOVE)
					dragSourceItem[0].dispose();
				dragSourceItem[0] = null; // should this be put into above conditional clause?
					
			}
		});

		DropTarget target = new DropTarget(tree, operations);
		target.setTransfer(types);
		target.addDropListener (new DropTargetAdapter() {
			Display display = PlatformUI.getWorkbench().getDisplay();
			public void dragOver(DropTargetEvent event) {
				event.feedback = DND.FEEDBACK_EXPAND | DND.FEEDBACK_SCROLL;
				if (event.item != null) {
					TreeItem item = (TreeItem)event.item;
					Point pt = display.map(null, tree, event.x, event.y);
					Rectangle bounds = item.getBounds();
					if (pt.y < bounds.y + bounds.height/3) {
						event.feedback |= DND.FEEDBACK_INSERT_BEFORE;
					} else if (pt.y > bounds.y + 2*bounds.height/3) {
						event.feedback |= DND.FEEDBACK_INSERT_AFTER;
					} else {
						event.feedback |= DND.FEEDBACK_SELECT;
					}
				}
			}
			public void drop(DropTargetEvent event) {
				
				if (event.data == null) {
					event.detail = DND.DROP_NONE;
					return;
				}
				Taxon taxon = (Taxon) dragSourceItem[0].getData();
				Taxon parentTaxon = (Taxon) event.item.getData();
				
				new ActionMoveTaxon(taxon, parentTaxon).run();
				
				// TODO choose appropriate parentTaxon depending on where in tree,
				//  esp. BETWEEN entries, taxon is dragged
				
//				if (event.item == null) {
//					// not sure which case this is ...
//					TreeItem item = new TreeItem(tree, SWT.NONE);
//					item.setText("unclear");
//				} else {
//					TreeItem item = (TreeItem)event.item;
//					Point pt = display.map(null, tree, event.x, event.y);
//					Rectangle bounds = item.getBounds();
//					TreeItem parent = item.getParentItem();
//					System.out.println(((Taxon) parent.getData()).getName().getNameCache());
//					if (parent != null) {
//						TreeItem[] items = parent.getItems();
//						int index = 0;
//						for (int i = 0; i < items.length; i++) {
//							if (items[i] == item) {
//								index = i;
//								break;
//							}
//						}
//						if (pt.y < bounds.y + bounds.height/3) {
//							TreeItem newItem = new TreeItem(parent, SWT.NONE, index);
//							newItem.setText(text);
//						} else if (pt.y > bounds.y + 2*bounds.height/3) {
//							TreeItem newItem = new TreeItem(parent, SWT.NONE, index+1);
//							newItem.setText(text);
//						} else {
//							TreeItem newItem = new TreeItem(item, SWT.NONE);
//							newItem.setText(text);
//						}
//						
//					} else {
//						System.out.println("y");
//						TreeItem[] items = tree.getItems();
//						int index = 0;
//						for (int i = 0; i < items.length; i++) {
//							if (items[i] == item) {
//								index = i;
//								break;
//							}
//						}
//						if (pt.y < bounds.y + bounds.height/3) {
//							TreeItem newItem = new TreeItem(tree, SWT.NONE, index);
//							newItem.setText(text);
//						} else if (pt.y > bounds.y + 2*bounds.height/3) {
//							TreeItem newItem = new TreeItem(tree, SWT.NONE, index+1);
//							newItem.setText(text);
//						} else {
//							TreeItem newItem = new TreeItem(item, SWT.NONE);
//							newItem.setText(text);
//						}
//					}
//				}
			}
		});		
	}

	/**
	 * Taxa with no NameCache - i.e. added with quick name -
	 *  can be edited in place in the tax. tree
	 */
	private void createQuickAdd() {
		
		final Tree tree = this.getTree();
		final TextCellEditor taxTreeNodeEditor = new TextCellEditor(tree);
		taxTreeNodeEditor.addListener(new ICellEditorListener() {
			public void applyEditorValue() {}
			public void cancelEditor() {
				// user has pressed ESC - transfer focus to parent item,
				//  remove taxon from tree
				Taxon taxon = (Taxon) tree.getSelection()[0].getData();
				tree.setSelection(tree.getSelection()[0].getParentItem());
				new ActionRemoveTaxonFromTree(taxon).run();
			}
			public void editorValueChanged(boolean oldValidState,
					boolean newValidState) {}
		});
		
		this.setCellEditors(new CellEditor[] {taxTreeNodeEditor});
		this.setColumnProperties(new String[] {"col1"});
		this.setCellModifier(new ICellModifier() {
			public boolean canModify(Object element, String property) {
				// If name element has not been initialized,
				//  this is a taxon added with QuickAdd
				if (((Taxon) element).getName().getTitleCache() == null)
					return true;
				return false;
			}
			public Object getValue(Object element, String property) {
				// If this node is editable, TitleCache is by definition empty
				return "";
			}
			public void modify(Object element, String property, Object value) {
				
				// Get Taxon object from TreeItem
				Taxon taxon = (Taxon)((Item) element).getData();
				
				if (((String) value).equals("")) {
					// Remove temporary Taxon from tree
					tree.setSelection(tree.getSelection()[0].getParentItem());
					new ActionRemoveTaxonFromTree(taxon).run(); 	
					
				} else {
					// Set Taxon's TitleCache, save it to CDM
					taxon.getName().setTitleCache((String) value);
					new ActionSaveTaxon(taxon).run();
				}
			}
		});
		
	}

	/**
	 * On double click, open name editor
	 */
	private void createDoubleClickListener() {
		this.addDoubleClickListener(new IDoubleClickListener(){

			public void doubleClick(DoubleClickEvent event) {								
				Taxon taxon = null;
				try {
					taxon = (Taxon) ((StructuredSelection)event.getSelection()).getFirstElement();
				}catch (Exception e){
					e.printStackTrace();
					taxon = null;
				}
				new ActionOpenNameEditor(taxon).run();
			}
		});		
	}

	/**
	 * Make right-click menu for tree nodes
	 */
	private void createRightClickListener() {
		
		final Tree tree = this.getTree();
		final Menu menu = new Menu(tree); 
		tree.setMenu(menu);
		menu.addMenuListener(new MenuAdapter() {
			public void menuShown(MenuEvent e) {
				
				// Get rid of existing menu items -
				//  same menu is re-used for all nodes
				MenuItem[] items = menu.getItems();
				for (int i = 0; i < items.length; i++) {
					((MenuItem) items[i]).dispose();
				}
				
				// Get Taxon associated with this node
				final Taxon taxon = (Taxon) tree.getSelection()[0].getData();
				System.out.println("open " + taxon.toString());
				
				// Add menu item to edit Taxon
				MenuItem itemEditTaxon = new MenuItem(menu, SWT.NONE);
				itemEditTaxon.setText("Edit taxon");
				itemEditTaxon.addSelectionListener(new SelectionListener() {
					public void widgetDefaultSelected(SelectionEvent e) {}
					public void widgetSelected(SelectionEvent e) {
						new ActionOpenNameEditor(taxon).run();
					}
				});

				new MenuItem(menu, SWT.SEPARATOR);
				
				// Add menu item to add a child node
				MenuItem itemAddChildTaxon = new MenuItem(menu, SWT.NONE);
				itemAddChildTaxon.setText("Add child taxon");
				itemAddChildTaxon.addSelectionListener(new SelectionListener() {
					public void widgetDefaultSelected(SelectionEvent e) {}
					public void widgetSelected(SelectionEvent e) {
						new ActionOpenNewChildNameEditor(taxon).run();
					}
				});
				
				// Add menu item to add a child node with edit name in place				
				MenuItem itemQuickAdd = new MenuItem(menu, SWT.NONE);
				itemQuickAdd.setText("Add child taxon with quick name");
				itemQuickAdd.addSelectionListener(new SelectionListener() {
					public void widgetDefaultSelected(SelectionEvent e) {}
					public void widgetSelected(SelectionEvent e) {
						new ActionAddQuickName(taxon).run();						
					}
				});
			}
		});
	}
	
	/**
	 * Set up content providers and viewer input
	 */
	private void createContent() {
		// Use custom content provider extended to add tree node retrieval
		ObservableListContentProvider viewerContentProviderList = new NameTreeContentProvider();
		this.setContentProvider(viewerContentProviderList);
				
		// Label provider that listens for changes to name cache
		IObservableMap[] viewerLabelProviderMaps = BeansObservables.observeMaps(viewerContentProviderList.getKnownElements(), 
				Taxon.class, new String[]{"name"});
		this.setLabelProvider(new ObservableMapLabelProvider(viewerLabelProviderMaps) {
			/* 
			 * TODO only show warning if there is something taxonomically amiss 
			 * 
			 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnImage(java.lang.Object, int)
			 */
			public Image getColumnImage(Object element, int columnIndex) {
				if (((Taxon) element).getName().getHasProblem())
					return ResourceManager.getPluginImage(Activator.getDefault(), "icons/warn_tsk.gif");
				return null;
			}
			/*
			 * JFace databinding syntax makes it to difficult to retrieve
			 * 	Taxon.getName().getNameCache, so override function that returns
			 *  label
			 * 
			 *  (non-Javadoc)
			 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnText(java.lang.Object, int)
			 */
			public String getColumnText(Object element, int columnIndex) {
				return ((Taxon) element).getName().getTitleCache();
			}				
		});
		
		// TaxonTreeList added to every time a node is opened with its
		//  children, or when a new taxon is added
		this.setInput(Activator.getDefault().getObservableTaxonTreeList());
		
		// Sort according to "getColumnText" above, i.e. by NameCache
		this.setComparator(new ViewerComparator());
	}

	/**
	 * If taxon node in the tree is hidden, open and select it
	 * 
	 * @param taxon
	 */
	public void revealTaxon(Taxon taxon) {
		this.setSelection(new StructuredSelection(taxon), true);
		this.reveal(taxon);
	}
	
	// Implement a "fake" tooltip
	final Listener labelListener = new Listener () {
		public void handleEvent (Event event) {
			Label label = (Label)event.widget;
			Shell shell = label.getShell ();
			switch (event.type) {
				case SWT.MouseDown:
					Event e = new Event ();
					e.item = (TableItem) label.getData ("_TABLEITEM");
					// Assuming table is single select, set the selection as if
					// the mouse down event went through to the table
					Tree tree = TaxonomicTreeViewer.this.getTree();
					tree.setSelection(new TreeItem [] {(TreeItem) e.item});
					tree.notifyListeners (SWT.Selection, e);
					shell.dispose ();
					tree.setFocus();
					break;
				case SWT.MouseExit:
					shell.dispose ();
					break;
			}
		}
	};	
	
	Listener tipListener = new Listener () {
		Shell tip = null;
		Label label = null;
		public void handleEvent (Event event) {
			switch (event.type) {
				case SWT.Dispose:
				case SWT.KeyDown:
				case SWT.MouseMove: {
					if (tip == null) break;
					tip.dispose ();
					tip = null;
					label = null;
					break;
				}
				case SWT.MouseHover: {
					Item item = TaxonomicTreeViewer.this.getItemAt(new Point(event.x, event.y));
					if (item != null) {
						if (tip != null  && !tip.isDisposed ()) tip.dispose ();
						tip = new Shell (Display.getCurrent(), SWT.ON_TOP | SWT.NO_FOCUS | SWT.TOOL);
						tip.setBackground (Display.getCurrent().getSystemColor (SWT.COLOR_INFO_BACKGROUND));
						FillLayout layout = new FillLayout ();
						layout.marginWidth = 2;
						tip.setLayout (layout);
						label = new Label (tip, SWT.NONE);
						label.setForeground (Display.getCurrent().getSystemColor (SWT.COLOR_INFO_FOREGROUND));
						label.setBackground (Display.getCurrent().getSystemColor (SWT.COLOR_INFO_BACKGROUND));
						label.setData ("_TABLEITEM", item);
//						label.setText (item.getText ());
						label.setText ("Joe mama");
						label.addListener (SWT.MouseExit, labelListener);
						label.addListener (SWT.MouseDown, labelListener);
						Point size = tip.computeSize (SWT.DEFAULT, SWT.DEFAULT);
						Rectangle rect = ((TreeItem) item).getBounds (0);
						Point pt = TaxonomicTreeViewer.this.getTree().toDisplay (rect.x, rect.y);
						tip.setBounds (pt.x, pt.y, size.x, size.y);
						tip.setVisible (true);
					}
				}
			}
		}
	};
	
}

