/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.map.IMapChangeListener;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.core.databinding.observable.map.MapChangeEvent;
import org.eclipse.core.databinding.observable.set.IObservableSet;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.editor.name.IterableSynonymyList;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author p.ciardelli
 * @created 29.05.2008
 * @version 1.0
 */
public class TaxonomicTreeLabelProvider extends ColumnLabelProvider {
	private static final Logger logger = Logger
			.getLogger(TaxonomicTreeLabelProvider.class);
	
	private final IObservableMap[] attributeMaps;

	private IMapChangeListener mapChangeListener = new IMapChangeListener() {
		public void handleMapChange(MapChangeEvent event) {
			Set affectedElements = event.diff.getChangedKeys();
			LabelProviderChangedEvent newEvent = new LabelProviderChangedEvent(
					TaxonomicTreeLabelProvider.this, affectedElements
							.toArray());
			fireLabelProviderChanged(newEvent);
		}
	};

	
	public TaxonomicTreeLabelProvider(IObservableSet observableTaxonSet) {
		this(BeansObservables.observeMaps(
				observableTaxonSet, 
				Taxon.class, new String[]{"name"}));
	}
	
	/**
	 * @param attributeMap
	 */
	public TaxonomicTreeLabelProvider(IObservableMap attributeMap) {
		this(new IObservableMap[] { attributeMap });
	}

	/**
	 * @param attributeMaps
	 */
	public TaxonomicTreeLabelProvider(IObservableMap[] attributeMaps) {
		System.arraycopy(attributeMaps, 0, this.attributeMaps = attributeMaps, 0, attributeMaps.length);
		for (int i = 0; i < attributeMaps.length; i++) {
			attributeMaps[i].addMapChangeListener(mapChangeListener);
		}
	}
	
	/** 
	 * 
	 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnImage(java.lang.Object, int)
	 **/
	public Image getColumnImage(Object element, int columnIndex) {
		if (element instanceof Taxon) {
			TaxonNameBase name = ((Taxon) element).getName();
			if (name != null && name.getHasProblem()) {
				return TaxEditorPlugin.getDefault().getImage(ITaxEditorConstants.WARNING_ICON);
			}
		}
		return null;
	}
	
	public String getText(Object element) {
		return getColumnText(element, 0);
	}
	
	/**
	 * JFace databinding syntax makes it to difficult to retrieve
	 * 	Taxon.getName().getNameCache, so override function that returns
	 *  label
	 * 
	 *  (non-Javadoc)
	 * @see org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider#getColumnText(java.lang.Object, int)
	 **/
	public String getColumnText(Object element, int columnIndex) {
		if (element instanceof Taxon) {
			return CdmUtil.getDisplayName((Taxon) element);					
		}
		return "";
	}	

	public String getToolTipText(Object element) {
		if (element instanceof Taxon) {
			return getSynonymyListDisplay((Taxon) element);
		}
		return null;
	}
	
	public void dispose() {
		for (int i = 0; i < attributeMaps.length; i++) {
			attributeMaps[i].removeMapChangeListener(mapChangeListener);
		}
		super.dispose();
	}
	
	/**
	 * @param taxon 
	 * @return
	 */
	private String getSynonymyListDisplay(Taxon taxon) {
		IterableSynonymyList synonymyList = new IterableSynonymyList(
				taxon);
		String synonymyListDisplay = CdmUtil
				.getDisplayName(taxon);
		
		for (TaxonBase synonymOrMisName : synonymyList) {
			TaxonNameBase name = synonymOrMisName.getName();
			if (name != null) {
				synonymyListDisplay += "\n      "
						+ CdmUtil.getDisplayName(name);
			}
		}
		return synonymyListDisplay;
	}
}
