// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.texteditor.AbstractDocumentProvider;

/**
 * Using an <code>IEditorPart</code>, creates a document where each line is associated
 * with an entity. Mapping between document positions and entities is stored in
 * <code>LineAnnotation</code>s in a <code>LineAnnotationModdel</code>.  
 * <p>
 * Requires:
 * <ul>
 * <li>an <code>IEntityCreator</code> to create entities for new lines;
 * <li>an <code>IEntityPersistenceService</code> for interacting with the persistence layer; and
 * <li>an <code>ILineDisplayStrategy</code> for various visual manifestations of the domain object. 
 * </ul> 
 * 
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 */
public class AnnotatedLineEditorDocumentProvider extends AbstractDocumentProvider {

	private static final Logger logger = Logger
			.getLogger(AnnotatedLineEditorDocumentProvider.class);
	
	private Map<IEditorInput, IDocument> documents = 
			new HashMap<IEditorInput, IDocument>();

	private Map<IEditorInput, IAnnotationModel> annotationModels = 
		new HashMap<IEditorInput, IAnnotationModel>();

	private Map<IEditorInput, IEntityCreator<?>> entityCreators = 
		new HashMap<IEditorInput, IEntityCreator<?>>();

	private Map<IEditorInput, IEntityPersistenceService> persistenceServices = 
		new HashMap<IEditorInput, IEntityPersistenceService>();

	private Map<IEditorInput, ILineDisplayStrategy> lineDisplayStrategies = 
		new HashMap<IEditorInput, ILineDisplayStrategy>();
	
	
	@Override
	public IAnnotationModel getAnnotationModel(Object element) {
		if (element instanceof IEditorInput) {
			
			IEditorInput input = (IEditorInput) element;
			IAnnotationModel model = annotationModels.get((IEditorInput) element);
			
			// Create model as necessary
			if (model == null) {
				model = new LineAnnotationModel(getLineDisplayStrategy(element));
				((LineAnnotationModel) model).setEntityCreator(getEntityCreator(element));
				annotationModels.put(input, model);
			}	
			return model;
		}
		
		return null;
	}
	
	@Override
	protected IAnnotationModel createAnnotationModel(Object element)
			throws CoreException {
		return getAnnotationModel(element);
	}

	@Override
	public IDocument getDocument(Object element) {
		if (element instanceof IEditorInput) {
			return documents.get((IEditorInput) element);
		}
		return null;
	}
	
	@Override
	protected IDocument createDocument(Object element) throws CoreException {
		
		if (element instanceof IEditorInput) {
			IEditorInput input = (IEditorInput) element;
			Document document = new Document("");
			IAnnotationModel model = getAnnotationModel(element);
					
			String text = "";
			String line;
			for (Object entity : getEntityList(element)) {
				
				Annotation annotation = new LineAnnotation(entity, getLineDisplayStrategy(element));
				
				line = annotation.getText()	+ document.getDefaultLineDelimiter();
				Position position = new Position(text.length(), line.length());
				model.addAnnotation(annotation, position);
				logger.info("Adding annotation " + entity);
				document.set(document.get() + line);
				
				
				
				text += line;
			}
			documents.put(input, document);
			if (model instanceof LineAnnotationModel) {
				((LineAnnotationModel) model).printAnnotations();
			}
			return document;
		}
		return null;
	}

	/**
	 * @param element 
	 * @return
	 */
	private List<?> getEntityList(Object element) {
		if (element instanceof IEditorInput) {
			return getPersistenceService(element).getEntityList(element);
		}
		return null;
	}

	@Override
	protected void doSaveDocument(IProgressMonitor monitor, Object element,
			IDocument document, boolean overwrite) throws CoreException {
		if (element instanceof IEditorInput) {
			
			IEntityPersistenceService persistenceService = getPersistenceService(element);
			
			// Get new containers from annotation model
			LineAnnotationModel model = (LineAnnotationModel) getAnnotationModel(element);
			Iterator iterator = model.getAnnotationIterator();
			while (iterator.hasNext()) {
				Annotation annotation = (Annotation) iterator.next();
				if (annotation instanceof IEntityContainer<?>) {
					IEntityContainer<?> container = (IEntityContainer<?>) annotation;
					if (container.isMarkedAsNew() || container.isDirty()) {
						persistenceService.save(container.getEntity(), element);
						container.setDirty(false);
						container.markAsNew(false);
					}
				}
			}
			for (LineAnnotation annotation : model.getDeletedAnnotations()) {
				if (annotation.isMarkedAsNew()) {
					continue;
				}
				if (annotation.isMarkedAsMerged()) {
					persistenceService.merge(annotation.getEntity(), annotation.getMergeTarget(), element);					
				}
				persistenceService.delete(annotation.getEntity(), element);
			}
			model.clearDeletedAnnotations();
		}
	}

	@Override
	protected IRunnableContext getOperationRunner(IProgressMonitor monitor) {
		return null;
	}
	
	@Override
	public boolean isModifiable(Object element) {
		return true;
	}
	
	@Override
	public boolean isReadOnly(Object element) {
		// enables copy & paste
		return false;
	}

	/**
	 * @param entityCreator
	 * @param element
	 */
	public void setEntityCreator(IEntityCreator<?> entityCreator, Object element) {
		if (element instanceof IEditorInput) {
			entityCreators.put((IEditorInput) element, entityCreator);
		}
	}
	
	/**
	 * @param element
	 * @return
	 */
	public IEntityCreator<?> getEntityCreator(Object element) {
		if (element instanceof IEditorInput) {
			IEntityCreator<?> entityCreator = entityCreators.get(element);
			Assert.isNotNull(entityCreator, "No IEntityCreator set for this element.");
			return entityCreator;
		}
		return null;
	}

	/**
	 * @param persistenceService
	 * @param input
	 */
	public void setPersistenceService(
			IEntityPersistenceService persistenceService, Object element) {
		if (element instanceof IEditorInput) {
			persistenceServices.put((IEditorInput) element, persistenceService);
		}
	}
	
	public IEntityPersistenceService getPersistenceService(Object element) {
		if (element instanceof IEditorInput) {
			IEntityPersistenceService persistenceService = persistenceServices.get((IEditorInput) element);
			Assert.isNotNull(persistenceService, "No IEntityPersistenceService set for this element.");
			return persistenceService;
		}
		return null;
	}

	/**
	 * @param lineDisplayStrategy
	 * @param input
	 */
	public void setLineDisplayStrategy(
			ILineDisplayStrategy lineDisplayStrategy, Object element) {
		if (element instanceof IEditorInput) {
			lineDisplayStrategies.put((IEditorInput) element, lineDisplayStrategy);
		}	
	}
	
	/**
	 * @param element
	 * @return
	 */
	private ILineDisplayStrategy getLineDisplayStrategy(Object element) {
		if (element instanceof IEditorInput) {
			ILineDisplayStrategy lineDisplayStrategy = lineDisplayStrategies.get((IEditorInput) element);
			Assert.isNotNull(lineDisplayStrategy, "No ILineDisplayStrategy set for this element.");
			return lineDisplayStrategy;
		}
		return null;
	}
}