/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.name.NameComposite;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * This class is supposed to handle possible duplications of cdm entities
 * for a specific editor
 * 
 * TODO break dependency to editor
 * 
 * @author n.hoffmann
 * @created 03.07.2009
 * @version 1.0
 *
 */
public class DuplicateArbitrator {
	private static final Logger logger = Logger.getLogger(DuplicateArbitrator.class);
	
	/**
	 * The editor this arbitrator should manage possible duplicates for
	 */
	private MultiPageTaxonEditor editor;
	private TaxonNameEditor nameEditor;
	private Set<NameComposite<TaxonBase>> dirtyNames;
	
	private Map<TaxonBase, ReferenceBase> duplicateReferences;

	private Map<TaxonBase, TaxonNameBase> duplicateNames;

	/**
	 * @param adaptableObject
	 */
	public DuplicateArbitrator(MultiPageTaxonEditor editor) {
		this.editor = editor;
		
		nameEditor = (TaxonNameEditor) editor.getPage(Page.NAME);
		
		duplicateReferences = new HashMap<TaxonBase, ReferenceBase>();
		duplicateNames = new HashMap<TaxonBase, TaxonNameBase>();
	}

	/**
	 * Provide strategies on how to handle possible duplications
	 */
	public void arbitrate() {
		// iterate over all names that were edited
		for(NameComposite composite : nameEditor.getDirtyNames()){
			logger.warn("Found " + composite + " with possible duplicates");
			
			// since we are dealing with NameComposites getData should always return a TaxonBase
			// also we do not want to handle viral names at the moment
//			NonViralName name = (NonViralName) ((TaxonBase) composite.getData()).getName();
			
			TaxonBase taxonBase = HibernateProxyHelper.deproxy(composite.getData(), TaxonBase.class);
			
			// TODO decide what entities are candidates for duplicate detection
			checkDuplicateLatinNames(taxonBase);
			checkDuplicateAuthors(taxonBase);
			checkDuplicateNomenclaturalReference(taxonBase);
			
			
		}
		
		solveDuplicates();
		
	}

	/**
	 * Provides strategies to solve found duplicates
	 * This can either happen automatically or through user input
	 */
	private void solveDuplicates() {
		/* 
		 * TODO first idea that comes to mind would be to present the user 
		 * a dialog with all possible duplicates and let her mark which entities 
		 * she wants to use
		 * 
		 * If we have sophisticated equals methods that guarantee full equality and 
		 * there is only one match of an entity we might want to resolve the duplication
		 * automatically.  
		 */		
		
		// debug
		for(ReferenceBase reference : duplicateReferences.values()){
			logger.warn(reference);
		}
		for(TaxonNameBase name : duplicateNames.values()){
			logger.warn(name);
		}
		
	}

	/**
	 * @param name
	 */
	private void checkDuplicateNomenclaturalReference(TaxonBase taxonBase) {
		
		NonViralName name = getName(taxonBase);
		
		try{
			ReferenceBase referenceBase = name.getNomenclaturalReference();
			
			// query datasource for the reference
			List<ReferenceBase> result = CdmStore.getReferenceService().getReferencesByTitle(referenceBase.getTitleCache());
			
			// if query delivers results, place possible duplicate in map
			for(ReferenceBase duplicateReference : result){
				duplicateReferences.put(taxonBase, duplicateReference);
			}
		}catch(NullPointerException e){
			logger.warn("Name has no nomenclatural reference");
		}
		
	}

	/**
	 * @param name
	 */
	private void checkDuplicateAuthors(TaxonBase taxonBase) {
		// see checkDuplicateNomenclaturalReference for implementation
	}

	/**
	 * @param name
	 */
	private void checkDuplicateLatinNames(TaxonBase taxonBase) {
		NonViralName name = getName(taxonBase);
		
		List<NonViralName> result = CdmStore.getNameService().findNamesByTitle(name.getTitleCache());
		
		for(Object object: result ){
			TaxonNameBase duplicateName = HibernateProxyHelper.deproxy(object, TaxonNameBase.class);
			duplicateNames.put(taxonBase, duplicateName);
		}
	}
	
	/**
	 * Helper method to get the NonViralName of a taxon base.
	 * 
	 * @param taxonBase
	 * @return
	 */
	private NonViralName getName(TaxonBase taxonBase){
		return HibernateProxyHelper.deproxy(taxonBase.getName(), NonViralName.class);
	}

}
