/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.FormToolkit;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.editor.description.CommonNameComposite;
import eu.etaxonomy.taxeditor.editor.description.DescriptionElementComposite;
import eu.etaxonomy.taxeditor.editor.description.DescriptionFeatureGroupComposite;
import eu.etaxonomy.taxeditor.editor.description.DescriptionGroupComposite;
import eu.etaxonomy.taxeditor.editor.description.DescriptionLabelComposite;
import eu.etaxonomy.taxeditor.editor.description.DistributionComposite;
import eu.etaxonomy.taxeditor.editor.description.DistributionFeatureGroupComposite;
import eu.etaxonomy.taxeditor.editor.description.TaxonDescriptionEditor;
import eu.etaxonomy.taxeditor.editor.images.GalleryLabelComposite;
import eu.etaxonomy.taxeditor.editor.images.ImageComposite;
import eu.etaxonomy.taxeditor.editor.images.ImagesGroupComposite;
import eu.etaxonomy.taxeditor.editor.images.TaxonImageEditor;
import eu.etaxonomy.taxeditor.editor.name.AcceptedNameComposite;
import eu.etaxonomy.taxeditor.editor.name.ConceptComposite;
import eu.etaxonomy.taxeditor.editor.name.ConceptGroupComposite;
import eu.etaxonomy.taxeditor.editor.name.HomotypicalGroupComposite;
import eu.etaxonomy.taxeditor.editor.name.MisappliedGroupComposite;
import eu.etaxonomy.taxeditor.editor.name.MisappliedNameComposite;
import eu.etaxonomy.taxeditor.editor.name.SynonymComposite;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;

/**
 * @author p.ciardelli
 * @created 08.01.2009
 * @version 1.0
 */
public class FreeTextElementFactory implements IFreeTextElementFactory {
	private static final Logger logger = Logger
			.getLogger(FreeTextElementFactory.class);
	
	private static IFreeTextElementFactory factory;
	
	public static IFreeTextElementFactory getDefault() {
		if (factory == null) {
			factory = new FreeTextElementFactory();
		}
		logger.trace("Created a new FreeTextElementFactory instance.");
		return factory;
	}
	
	public Composite createAcceptedTaxon(AbstractTaxonEditor editor) {
		// Create a homotypic group composite for the accepted taxon
		HomotypicalGroup group = editor.getTaxon().getHomotypicGroup();
		Assert.isNotNull(group, "Taxon does not have a homotypic group");
		Composite groupComposite = createHomotypicalGroup(editor, group);
				
		// Create a name composite for the accepted taxon
		Composite acceptedNameComposite = new AcceptedNameComposite(editor, groupComposite);
	
		// Return the name composite
		return acceptedNameComposite;
	}
	
	public Composite createSynonym(TaxonNameEditor editor, Synonym synonym) {		
		// Get the synonym's homotypic group
		HomotypicalGroup group = synonym.getHomotypicGroup();
		Assert.isNotNull(group, "Synonym does not have a homotypic group");
		
		// If the group doesn't yet have a composite, create one and add it to the repository
		Composite groupComposite = editor.getHomotypicGroup(group);
		if (groupComposite == null) {
			groupComposite = createHomotypicalGroup(editor, group);
		}
				
		// Create a synonym composite in the homotypical group
		Composite synonymComposite = new SynonymComposite(editor, groupComposite, synonym);
				
		return synonymComposite;
	}
	
	
	public Composite createMisappliedName(TaxonNameEditor editor, Taxon misappliedName) {
		
		// If there is no composite for misapplied names, 
		//	create one and add it to the repository
		Composite groupComposite = editor.getMisappliedGroup();
		if (groupComposite == null) {
			groupComposite = createMisappliedGroup(editor);
		}
		
		// Create the name's composite
		Composite composite = new MisappliedNameComposite(editor, groupComposite, misappliedName);
		
		return composite;
	}
	
	
	public Composite createMisappliedGroup(TaxonNameEditor editor) {
		// Create the group composite
		Composite groupComposite = new MisappliedGroupComposite(editor, editor.getTopLevelComposite());
		groupComposite.addFocusListener(new CompositeBorderDecorator(groupComposite, editor.getManagedForm()));
		
		// Put the group composite before concept group composite, if any
		Composite conceptGroupComposite = editor.getConceptGroup();
		if (conceptGroupComposite != null) {
			groupComposite.moveAbove(conceptGroupComposite);
		}
		
		return groupComposite;
	}
	
	public Composite createHomotypicalGroup(AbstractTaxonEditor editor, HomotypicalGroup group) {

		// Create the group composite
		Composite groupComposite = new HomotypicalGroupComposite(editor, editor.getTopLevelComposite(), group);
//		groupComposite.addFocusListener(new CompositeBorderDecorator(groupComposite, editor.getManagedForm()));		
				
		groupComposite.setData(FormToolkit.KEY_DRAW_BORDER, FormToolkit.TEXT_BORDER);
		editor.getManagedForm().getToolkit().paintBordersFor(groupComposite.getParent());
		
		return groupComposite;
	}
	
	public Composite createConcept(TaxonNameEditor editor, TaxonRelationship relationship) {	
		// If there is no composite for misapplied names, 
		//	create one and add it to the repository
		Composite groupComposite = editor.getConceptGroup();
		if (groupComposite == null) {
			groupComposite = createConceptGroup(editor);
		}
		
		// Create the name's composite
		Composite composite = ConceptComposite.getNewInstance(editor, groupComposite, relationship);
				
		return composite;
	}

	
	public Composite createConceptGroup(TaxonNameEditor editor) {
		// Create the group composite
		Composite groupComposite = new ConceptGroupComposite(editor, editor.getTopLevelComposite());
		groupComposite.addFocusListener(new CompositeBorderDecorator(groupComposite, editor.getManagedForm()));
		
		// Put the group composite after misapplied group composite, if any
		Composite misappliedGroupComposite = editor.getMisappliedGroup();
		if (misappliedGroupComposite != null) {
			groupComposite.moveBelow(misappliedGroupComposite);
		}
		
		return groupComposite;

	}

	
	/*********************** DESCRIPTION **********************************/ 
	
	public Composite createDescriptionLabel(AbstractTaxonEditor editor, TaxonDescription description) {
		
		// Create a group composite for the description
		Composite groupComposite = createDescriptionGroup(editor, description);
				
		// Create label composite
		Composite labelComposite =
				new DescriptionLabelComposite(editor, groupComposite, description);
	
		// Return the label composite
		return labelComposite;
	}

	
	public Composite createDescriptionElement(TaxonDescriptionEditor editor, TaxonDescription description,
			DescriptionElementBase element) {

		// Get the element's feature
		Feature feature = HibernateProxyHelper.deproxy(element.getFeature(), Feature.class);
		Assert.isNotNull(feature, "Element does not have a feature.");
							
		// If the feature doesn't yet have a composite, create one
		Composite groupComposite = editor.getDescriptionFeatureGroup(description, feature);
		if (groupComposite == null) {
			groupComposite = createDescriptionFeatureGroup(editor, description, feature);
		}
				
		// Create an element composite in the feature group according to its type
		Composite elementComposite = null;
		if (element.isInstanceOf(CommonTaxonName.class)) {
			elementComposite =
					new CommonNameComposite(editor, groupComposite, (CommonTaxonName) element);
		}else if(element.isInstanceOf(Distribution.class)){
			elementComposite =
					new DistributionComposite(editor, groupComposite, (Distribution) element);
		} else {
			elementComposite = 
					new DescriptionElementComposite(editor, groupComposite, element);
		}
		
		return elementComposite;
	}

	
	public Composite createDescriptionGroup(AbstractTaxonEditor editor,
			TaxonDescription description) {
		// Create the group composite
		GroupComposite groupComposite = new DescriptionGroupComposite(editor.getTopLevelComposite(), editor, description);
		groupComposite.addFocusListener(new CompositeBorderDecorator(groupComposite, editor.getManagedForm()));
//		groupComposite.setDroppable(false);
				
		return groupComposite;
	}
		
	
	public Composite createDescriptionFeatureGroup(TaxonDescriptionEditor editor,
			TaxonDescription description, Feature feature) {

		// Description label should be created before adding features
		Composite descriptionGroupComposite = editor.getDescriptionGroup(description);
		Assert.isNotNull(descriptionGroupComposite, 
				"Description group must be created - usually by creating its label - before adding features.");
		
		// Create the group composite
		Composite groupComposite;
		if(feature.equals(Feature.DISTRIBUTION())){
			groupComposite = new DistributionFeatureGroupComposite
										(editor, descriptionGroupComposite, description, feature);
		}else{
			groupComposite= new DescriptionFeatureGroupComposite
										(editor, descriptionGroupComposite, description, feature);
		}
		groupComposite.addFocusListener(new CompositeBorderDecorator(groupComposite, editor.getManagedForm()));	
		
		return groupComposite;		
	}
	
	/********************* IMAGES *************************/
	

	public Composite createGalleryLabel(AbstractTaxonEditor editor, TaxonDescription imageGallery) {
		
		// Create a group composite for the description
		Composite groupComposite = createDescriptionGroup(editor, imageGallery);
				
		// Create label composite
		Composite labelComposite =
				new GalleryLabelComposite(editor, groupComposite, imageGallery);
	
		// Return the label composite
		return labelComposite;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.IFreeTextElementFactory#createImageElement(org.eclipse.ui.forms.ManagedForm, eu.etaxonomy.cdm.model.taxon.Taxon, java.net.URL)
	 */
	public Composite createImageElement(TaxonImageEditor editor, ImageFile imageFile) {

		// Get the group composite for all images
		GroupComposite groupComposite = editor.getImageGroup();
		if (groupComposite == null) {
			groupComposite = createImagesGroup(editor);
		}
		
		// Create the image composite
		GroupedComposite imageComposite = new ImageComposite(editor, groupComposite, imageFile);
		imageComposite.addFocusListener(new CompositeBorderDecorator(imageComposite, editor.getManagedForm()));
				
		return imageComposite;
	}

	/**
	 * @param taxon
	 * @param form2 
	 * @return
	 */
	private GroupComposite createImagesGroup(AbstractTaxonEditor editor) {

		// Create the group composite
		GroupComposite groupComposite = new ImagesGroupComposite(editor.getTopLevelComposite(), editor);		
				
		return groupComposite;
	}
}
