/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.description;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.taxeditor.editor.CompositeBorderDecorator;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.editor.LineWrapSupport;
import eu.etaxonomy.taxeditor.editor.ParseListener;
import eu.etaxonomy.taxeditor.model.DescriptionHelper;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.propertysheet.description.DescriptionElementPropertySource;

/**
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class CopyOfDescriptionElementComposite extends GroupedComposite {
	private static final Logger logger = Logger.getLogger(CopyOfDescriptionElementComposite.class);

	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public static final String TEXT_DATA = "text_data";
	public static final String COMMON_TAXON_NAME = "common_taxon_name";
	public static final String DISTRIBUTION = "distribution";

	/**
	 * ************ FONTS ************
	 */
	public static final Font ELEMENT_FONT = Resources
			.getFont(Resources.SYNONYM_FONT);

	/**
	 * ************ ICONS ************
	 */
	public static final Image BLACK_SQUARE_ICON = ImageResources
			.getImage(ImageResources.BLACK_SQUARE_ICON);

	protected DescriptionElementBase element;
	private DescriptionViewer descriptionViewer;

	private static final String EMPTY_NAME_PROMPT = "Click to add descriptive element text";
	
	private static final int STYLES = SWT.WRAP | SWT.MULTI | SWT.RESIZE;

	public CopyOfDescriptionElementComposite(TaxonDescriptionEditor editor, Composite parent, DescriptionElementBase element) {
		super(editor, parent);
		
		this.element = element;
				
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();
		
		createParser();
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		setIcon(BLACK_SQUARE_ICON);
		setFont(ELEMENT_FONT);

		setMenu(editor.getMenu());
		
		initTextViewer();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#createTextViewer()
	 */
	@Override
	protected void createTextViewer() {
		descriptionViewer = new DescriptionViewer(this, STYLES, managedForm);
		descriptionViewer.setBackground(getBackground());
		
		FocusAdapter focusListener = new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				setFocus();
			}
		};		
		descriptionViewer.addFocusListener(focusListener);
		
		MouseAdapter mouseListener = new MouseAdapter() {
			public void mouseDown(MouseEvent e) {
				setFocus();				
			}
		};
		this.addMouseListener(mouseListener);
		descriptionViewer.addMouseListener(mouseListener);
	}
	
	protected void initTextViewer() {
		
		String text = null;

		text = DescriptionHelper.getCache(element);
		
		if (text.length() == 0) {
			initEmptyText();
		} else {
			descriptionViewer.setText(text);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#createEmptyViewerPrompt(java.lang.String)
	 */
	@Override
	public void createEmptyViewerPrompt(final String prompt) {
		 
		final Font promptFont = Resources
				.getFont(Resources.FONT_DEFAULT_PROMPT);
		FocusListener focusListener = new FocusListener() {
			
			public void focusGained(FocusEvent e) {
				if (descriptionViewer.getText().equals(EMPTY_NAME_PROMPT)) {
					descriptionViewer.setFont(getViewerFont());
					descriptionViewer.setText("");
				}
			}
			
			public void focusLost(FocusEvent e) {
				if (descriptionViewer.getText().length() == 0) {
					initEmptyText();
				}
			}
			
		};
		setFocusListener(focusListener);
		descriptionViewer.addFocusListener(focusListener);
		
		if (descriptionViewer.getText().length() == 0) {
			descriptionViewer.setFont(promptFont);
			descriptionViewer.setText(prompt);
		}
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#initEmptyText()
	 */
	@Override
	protected void initEmptyText() {
		descriptionViewer.setFont(
				Resources
					.getFont(Resources.FONT_DEFAULT_PROMPT));
		
		descriptionViewer.setText(EMPTY_NAME_PROMPT);
//		textViewer.setCursorToEOL();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#setIcon(org.eclipse.swt.graphics.Image)
	 */
	@Override
	public void setIcon(Image icon) {
		descriptionViewer.setIcon(icon);
	}
	
	public DescriptionElementBase getElement() {
		return element;
	}
	
	protected void parse(String text) {
		
		DescriptionHelper.setCache(getElement(), text);
		
		// Manually refresh the property sheet to reflect changes
		setSelection();
		
		setDirty(true);
	}
	
	public IPropertySource getPropertySource() {
		return new DescriptionElementPropertySource(getElement());
	}

	@Override
	protected Font getViewerFont() {
		return ELEMENT_FONT;
	}
	
	public Object getData () {
		return getElement();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#setFont(org.eclipse.swt.graphics.Font)
	 */
	@Override
	public void setFont(Font font) {
		descriptionViewer.setFont(font);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#createParser()
	 */
	@Override
	protected void createParser() {
		ParseListener parseListener = new ParseListener() {
			@Override
			public void parse(String text) {
				if (!EMPTY_NAME_PROMPT.equals(text)) {
					CopyOfDescriptionElementComposite.this.parse(text);
				}
			}
		};
		descriptionViewer.addModifyListener(parseListener);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#setMenu(org.eclipse.swt.widgets.Menu)
	 */
	@Override
	public void setMenu(Menu menu) {
		descriptionViewer.setMenu(menu);
		super.setMenu(menu);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#createBorderSupport()
	 */
	@Override
	public void createBorderSupport() {
//		CompositeBorderDecorator borderDecorator = new CompositeBorderDecorator(
//				descriptionViewer, managedForm);
		CompositeBorderDecorator borderDecorator = new CompositeBorderDecorator(
				this, managedForm);			
//		borderDecorator.setLoseFocus(true);
		descriptionViewer.addFocusListener(borderDecorator);
		setBorderDecorator(borderDecorator);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#createLineWrapSupport()
	 */
	@Override
	protected void createLineWrapSupport() {
		new LineWrapSupport(descriptionViewer, managedForm);
	}

	/**
	 * @param text
	 */
	protected void setText(String text) {
		descriptionViewer.setText(text);
	}

	/**
	 * @param editable
	 */
	protected void setEditable(boolean editable) {
		descriptionViewer.setEditable(editable);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#setBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setBackground(Color color) {
		if (descriptionViewer != null) {
			descriptionViewer.setBackground(color);
		}
		super.setBackground(color);
	}
}