// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.api.service.ILocationService.NamedAreaVocabularyType;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.reference.ReferenceSelectComposite;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 03.04.2009
 * @version 1.0
 */
public class DistributionDialog extends Dialog {
	
	private ILocationService locationService;

	private String title;
	private String message;
	
	/**
	 * Number of area menus to show
	 */
	private static final int NUM_AREA_MENUS = 4; 
	/**
	 * UI elements
	 */
	private Combo vocabTypeCombo;
	private Combo[] areaCombos = new Combo[NUM_AREA_MENUS];
	private Combo statusCombo;
	private ReferenceSelectComposite referenceComposite;
	/**
	 * Distribution fields
	 */
	private NamedArea selectedArea;
	private PresenceAbsenceTermBase<?> selectedStatus;
	private Distribution distribution;
	private NamedAreaVocabularyType selectedVocabType;
	private ReferenceBase<?> selectedReference;
	/**
	 * Lists to populate drop downs
	 */
	private ArrayList<NamedAreaVocabularyType> vocabularyTypes;  
	private List<PresenceAbsenceTermBase<?>> presenceAbsenceTerms;
	private List<List<NamedArea>> areas = new ArrayList<List<NamedArea>>();
	private NamedArea[] selectedAreas = new NamedArea[NUM_AREA_MENUS];
	/**
	 * Static fields to persist default values
	 */
	private static ReferenceBase<?> lastReference;
	private static PresenceAbsenceTermBase<?> lastPresenceAbsence;
	private static NamedAreaVocabularyType lastVocabType;
	private static NamedArea lastNamedArea;

	/**
	 * @param shell
	 * @param dialogTitle
	 * @param dialogMessage
	 */
	public DistributionDialog(Shell shell, String dialogTitle,
			String dialogMessage) {
		super(shell);
	
		title = dialogTitle;
		message = dialogMessage;

		locationService = CdmStore.getLocationService();
		
		recallPreviousSelections();
	}

	/**
	 * Set inputs to selections from last time dialog was used.
	 */
	private void recallPreviousSelections() {
		if (lastVocabType != null)  {
			selectedVocabType = lastVocabType;

			// Last named area don't make much sense w/out a vocab type
			if (lastNamedArea != null) {
				selectedArea = lastNamedArea;
			}
		}
		if (lastPresenceAbsence != null) {
			selectedStatus = lastPresenceAbsence; 
		}
		if (lastReference != null) {
			selectedReference = lastReference;
		}
	}

	/**
	 * @param dialogTitle 
	 * @param dialogMessage 
	 * @return
	 */
	public static Distribution getDistribution(String dialogTitle,
			String dialogMessage) {
		DistributionDialog dialog = new DistributionDialog(EditorUtil.getShell(), 
				dialogTitle, 
				dialogMessage);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		return dialog.getDistribution();
	}

	/**
	 * @return
	 */
	private Distribution getDistribution() {
		return distribution;
	}
	
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        if (title != null) {
			shell.setText(title);
		}
    }
	
    protected Control createDialogArea(Composite parent) {
        // create composite
    	Composite composite = (Composite) super.createDialogArea(parent);
        // create message
        if (message != null) {
            createLabel(message, composite, parent);
        }
    	createLabel("Select an area vocabulary:", composite, parent);
        // create vocab type
        vocabTypeCombo = createCombo(composite);
        vocabTypeCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectVocabType();
			}
		});
        // create areas
        createLabel("Select an area:", composite, parent);
        for (int i = 0; i < NUM_AREA_MENUS; i++) {
        	areaCombos[i] = createCombo(composite); 
            areaCombos[i].addSelectionListener(new AreaComboListener(i));
            areaCombos[i].setEnabled(false);
        }
        // create presence / absence
        createLabel("Select status:", composite, parent);
		statusCombo =  new Combo(composite, SWT.BORDER);
		statusCombo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
		statusCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectStatusTerm();
			}
		});
		// create reference
		referenceComposite = new ReferenceSelectComposite(composite);

        populateVocabTypes();
		populateAreas();
        
		populatePresenceAbsenceTerms();
		populateReference();
        
        applyDialogFont(composite);
        
        return composite;
    }

	class AreaComboListener extends SelectionAdapter {
    	private int i;
		AreaComboListener(int i) {
    		this.i = i;
    	}
		@Override
		public void widgetSelected(SelectionEvent e) {
			selectArea(i);
		}
    }
    
	/**
	 * @return
	 */
	private Combo createCombo(Composite composite) {
		Combo combo = new Combo(composite, SWT.BORDER);
		combo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
				| GridData.HORIZONTAL_ALIGN_FILL));
		return combo;
	}
    
	/**
	 * 
	 */
	private void populateVocabTypes() {
		int i = 0;		
		int index = -1;
		for (NamedAreaVocabularyType vocabularyType : getVocabularyTypes()) {
			String vocabularyTypeLabel = vocabularyType == null ? 
					"" : vocabularyType.toString();
			vocabTypeCombo.add(vocabularyTypeLabel);
			
			if (vocabularyType != null && vocabularyType == selectedVocabType) {
				index = i;
			}
			i++;
		}
		if (index > -1) {
			vocabTypeCombo.select(index);
		}
	}
	
	/**
	 * @param namedArea
	 * @return
	 */
	private List<NamedArea> getNamedAreas(NamedArea namedArea, int depth) {
		List<NamedArea> terms = new ArrayList<NamedArea>();
//		Set<NamedArea> terms = namedArea.getIncludes();
		terms.addAll(namedArea.getIncludes());
		
//		// FIXME fixme how?!
//		List<NamedArea> termList = new ArrayList<NamedArea>();
//		for (NamedArea area : terms){
//			termList.add(area);
//		}
		
		areas.add(depth, terms);
		
		return terms;
	}

	/**
	 * @param selectedVocabType
	 * @return
	 */
	private List<NamedArea> getNamedAreas(NamedAreaVocabularyType vocabularyType) {		
		List<NamedArea> topLevelTerms = locationService.getTopLevelNamedAreasByVocabularyType(vocabularyType);
		
//		OrderedTermVocabulary<NamedArea> vocabulary = locationService.getNamedAreaVocabulary(vocabularyType);
//
//		List<NamedArea> topLevelTerms = new ArrayList<NamedArea>();
//		Iterator<NamedArea> it = vocabulary.iterator();
//		while(it.hasNext()){
//			
//			NamedArea area =  HibernateProxyHelper.deproxy(it.next(), NamedArea.class);
////			try{
////				area3 =
////			}catch(Exception e){
////				logger.warn("Nope");
////			}
////			NamedArea area2 = area3.getPartOf();
////			
////			if(!(area2 instanceof NamedArea)){
////				 area2 = CdmBase.deproxy(area2, NamedArea.class);
////			}
//			
//			if(area.getPartOf() == null){
//				topLevelTerms.add(area);
//			}
//
//		}
		// TODO is this the right order?
		Collections.reverse(topLevelTerms);
		
		areas.add(0, topLevelTerms);
		return topLevelTerms;
	}
//		for(NamedArea area : vocabulary){	
//			NamedArea area2 = area.getPartOf();
//			
//			if(!(area.getPartOf() instanceof NamedArea)){
//				 area2 = CdmBase.deproxy(area.getPartOf(), NamedArea.class);
//			}
//			
//			if(area2 == null){
//				topLevelTerms.add(area);
//			}


	/**
	 * 
	 */
	private void populatePresenceAbsenceTerms() {
		int selectedIndex = -1;
		int i = 0;
		for (PresenceAbsenceTermBase<?> term : getPresenceAbsenceTerms()) {
			if (term == null) {
			} else {
				statusCombo.add(term.getLabel(CdmStore.getDefaultLanguage()));				
				if (this.selectedStatus != null && this.selectedStatus.equals(term)) {
					selectedIndex = i;
				}
			}
			i++;
		}
		if (selectedIndex != -1) {
			statusCombo.select(selectedIndex);	
		}
	}

	/**
	 * 
	 */
	private void populateReference() {
		if (selectedReference != null) {
			referenceComposite.setReference(selectedReference);
		}
	}
	
	/**
	 * @return
	 */
	private List<NamedAreaVocabularyType> getVocabularyTypes() {
		if (vocabularyTypes == null) {
			vocabularyTypes = new ArrayList<NamedAreaVocabularyType>();
			vocabularyTypes.addAll(locationService.getNamedAreaVocabularyTypes());
		}
		return vocabularyTypes;
	}
	
	/**
	 * @return
	 */
	private List<PresenceAbsenceTermBase<PresenceAbsenceTermBase<?>>> getPresenceAbsenceTerms() {
		return PreferencesUtil.getPreferredPresenceAbsenceTerms();
	}

	/**
	 * @param i
	 */
	protected void selectArea(int depth) {
		// This selection is made currently selected area
		int i = areaCombos[depth].getSelectionIndex();
		
		selectedArea = getSelectedArea(depth, i);
		
		selectedAreas[depth] = selectedArea;
		
		clearArea(depth + 1);
		populateAreas(depth + 1);
				
		updateButtons();
	}

	/**
	 * @param i
	 * @param index
	 * @return
	 */
	private NamedArea getSelectedArea(int depth, int index) {
		return areas.get(depth).get(index);
	}

	/**
	 * 
	 */
	protected void selectVocabType() {
		int i = vocabTypeCombo.getSelectionIndex();
		selectedVocabType = getVocabularyTypes().get(i);
		
		clearAreas();
		
		populateAreas(0);
	}

    /**
	 * 
	 */
	private void populateAreas() {
		if (selectedArea == null) {
			return;
		}
		List<NamedArea> areasTmp = new ArrayList<NamedArea>();
		NamedArea namedAreaTmp = selectedArea;
		NamedArea parentArea = null;
		do {
			areasTmp.add(namedAreaTmp);
			parentArea = namedAreaTmp.getPartOf();
			namedAreaTmp = parentArea;
		} while (parentArea != null);
		Collections.reverse(areasTmp);
		int depth = 0;
		for (NamedArea namedArea : areasTmp) {
			selectedAreas[depth] = namedArea;
			populateAreas(depth);
			depth++;
		}
	}
	
	/**
	 * @param i
	 */
	private void populateAreas(int depth) {
		List<NamedArea> areasTmp;
		if(depth == 0){
		  	 areasTmp = getNamedAreas(selectedVocabType);
		}else{
			areasTmp = getNamedAreas(selectedAreas[depth - 1], depth);
		}
		
		int index = -1;
		int i = 0;
		if (areasTmp.size() > 0) {
			for(NamedArea areaTmp : areasTmp){
				String areaLabel = areaTmp == null ? "" : areaTmp.getLabel();
				areaCombos[depth].add(areaLabel);
				
				if (selectedAreas[depth] != null && selectedAreas[depth].equals(areaTmp)) {
					index = i;
				}
				i++;
			}
			areaCombos[depth].setEnabled(true);
			if (index > -1) {
				areaCombos[depth].select(index);
			}
		}
	}

	/**
	 * 
	 */
	protected void selectStatusTerm() {
		int i = statusCombo.getSelectionIndex();
		selectedStatus = presenceAbsenceTerms.get(i);
		
		updateButtons();
	}

	/**
	 * @param string
	 * @param parent 
	 * @param composite 
	 * @return 
	 */
	private Label createLabel(String string, Composite composite, Composite parent) {
		Label label = new Label(composite, SWT.WRAP);
        label.setText(string);
        GridData data = new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER);
        data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
        label.setLayoutData(data);
        label.setFont(parent.getFont());
        return label;
	}

	/**
	 * @param i
	 */
	private void clearArea(int i) {
		for (int j = i; j < NUM_AREA_MENUS; j++) {
			// TODO add clear selectedArea
			areaCombos[j].removeAll();
			areaCombos[j].setEnabled(false);
		}
	}

	/**
	 * 
	 */
	private void clearAreas() {
		clearArea(0);
	}
	
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
        
        updateButtons(); 
    }
	
	protected void buttonPressed(int buttonId) {
		if (IDialogConstants.OK_ID == buttonId) {
			okPressed();
		} else if (IDialogConstants.CANCEL_ID == buttonId) {
			cancelPressed();
		}
	}

	protected void okPressed() {
		createDistribution();
		setStaticFields();
		super.okPressed();
	}
	
	/**
	 * 
	 */
	private void setStaticFields() {
		lastPresenceAbsence = getPresenceAbsenceTerm();
		lastNamedArea = selectedArea;
		lastReference = getReference(); 
		lastVocabType = getVocabType();
	}

	/**
	 * @return
	 */
	private NamedAreaVocabularyType getVocabType() {
		int i = vocabTypeCombo.getSelectionIndex();
		return vocabularyTypes.get(i);
	}

	/**
	 * @return
	 */
	private ReferenceBase<?> getReference() {
		return referenceComposite.getReference();
	}

	/**
	 * @return
	 */
	private String getMicroReference() {
		return referenceComposite.getMicroReference();
	}
	
	/**
	 * @return
	 */
	private PresenceAbsenceTermBase<?> getPresenceAbsenceTerm() {
		int i = statusCombo.getSelectionIndex();
		return presenceAbsenceTerms.get(i);
	}
	
	/**
	 * 
	 */
	private void createDistribution() {
		distribution = Distribution.NewInstance(selectedArea, selectedStatus);
		if (referenceComposite.getReference() != null) {
			selectedReference = getReference();
			distribution.setCitation(selectedReference);
		}
		if (referenceComposite.getMicroReference() != "") {
			distribution.setCitationMicroReference(
					getMicroReference());
		}
	}

	private void updateButtons() {
		setOkEnabled(isDialogComplete());
	}
		
	private boolean isDialogComplete() {
		return
		 	(selectedArea != null && selectedStatus != null);
	}
	
	private void setOkEnabled(boolean enabled) {
		getButton(IDialogConstants.OK_ID).setEnabled(enabled);
	}
}