/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name;

import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.editor.labels.ILabelImageStrategy;
import eu.etaxonomy.taxeditor.editor.labels.LabelImageProvider;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.operations.CreateSynonymInNewGroupOperation;
import eu.etaxonomy.taxeditor.propertysheet.name.BotanicalNamePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.name.NonViralNamePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.name.ZoologicalNamePropertySource;

/**
 * Formats an <code>GroupedComposite</code> to display <code>TaxonNameBase</code> elements
 * in a <code>NameViewer</code>.
 * 
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public abstract class NameComposite<T extends TaxonBase> extends GroupedComposite{

	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public String compositeType;
	public static final String ACCEPTED_TAXON = "accepted_name_composite";
	public static final String HOMOTYPIC_SYNONYM = "homotypic_name_composite";
	public static final String HETEROTYPIC_SYNONYM = "heterotypic_name_composite";
	public static final String MISAPPLIED_NAME = "misappliedname_name_composite";
	public static final String CONCEPTRELATION = "concept_name_comcposite";

	/**
	 * ************ INDENTATIONS ************
	 */
	public static final int ACCEPTED_INDENT = 0;
	public static final int SYNONYM_INDENT = 15;
	public static final int MISAPPLIEDNAME_INDENT = 15;
	public static final int CONCEPT_INDENT = 15;

	/**
	 * ************ FONTS ************
	 */
	public static final Font ACCEPTED_FONT = Resources
			.getFont(Resources.ACCEPTED_TAXON_FONT);
	public static final Font SYNONYM_FONT = Resources
			.getFont(Resources.SYNONYM_FONT);
	public static final Font MISAPPLIEDNAME_FONT = Resources
			.getFont(Resources.MISAPPLIEDNAME_FONT);
	public static final Font CONCEPT_FONT = Resources
			.getFont(Resources.CONCEPT_FONT);

	/**
	 * ************ ICONS ************
	 */
	public static final Image ACCEPTED_ICON = ImageResources
			.getImage(ImageResources.BLACK_SQUARE_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ICON = ImageResources.getImage(ImageResources.HOMOTYPIC_SYN_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = ImageResources.getImage(
			ImageResources.HOMOTYPIC_SYN_ORIGINAL_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ICON = ImageResources.getImage(ImageResources.HETEROTYPIC_SYN_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = ImageResources.getImage(
			ImageResources.HETEROTYPIC_SYN_ORIGINAL_ICON);
	public static final Image MISAPPLIEDNAME_ICON = ImageResources.getImage(ImageResources.MISAPPLIED_NAME_ICON);
	public static final Image CONCEPT_ICON = ImageResources.getImage(ImageResources.CONCEPT_ICON);
	public static final Image AUTONYM_ICON = ImageResources
			.getImage(ImageResources.AUTONYM_ICON);
	public static final Image MOVE = ImageResources.getImage(ImageResources.MOVE_ICON);

	/**
	 * ************ TRANSFORMATIONS ************
	 */
	public static final String ADD_GROUP_BASIONYM = "add_group_basionym";
	public static final String REMOVE_GROUP_BASIONYM = "remove_group_basionym";

	/**
	 * ************ MENU ACTIONS ************
	 */
	public Action CHANGE_TAXON_TO_SYNONYM_ACTION;

	private static final String EMPTY_NAME_PROMPT = "Click to add name";

	/**
	 * Used to turn parser on and off.
	 * 
	 * @see activateParser
	 * @see deactivateParser
	 */
	private boolean isUseParser = false;

	protected boolean isParsing;
	
	/**
	 * Whether this NameComposite has unsaved changes 
	 */
	private boolean isDirty = false;
	
	private T taxonBase;
	
	/**
	 * The constructor for a DescriptionElementComposite. Takes a parent Composite on which to
	 * create itself, and an IManagedForm for Composite life cycle methods, i.e.
	 * drawing borders, creating other Composites, creating line wrap support,
	 * etc.
	 * 
	 * @param parent
	 * @param managedForm
	 */
	public NameComposite(AbstractTaxonEditor editor, Composite parent, T taxonBase) {
		super(editor, parent);

		this.taxonBase = taxonBase;
		
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();
		createLineBreakListener();

		this.setMenu(editor.getMenu());
		
		setDraggableControl(textViewer.getRulerControl());
		
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		initializeComposite();
		createParser();
	}
	
	/**
	 * Initialize the composite specific code
	 * 
	 */
	protected abstract void initializeComposite();
	
	protected String getEmptyTextPrompt() {
		return EMPTY_NAME_PROMPT;
	}
	
	/**
	 * 
	 */
	private void showNameRelations() {
		TaxonNameBase<?, ?> name = getName();
		if (name == null) {
			return;
		}
		
		ILabelImageStrategy strategy = LabelImageProvider.getLabelStrategy(name);
		LabelImageProvider labelProvider = new LabelImageProvider(strategy);
		
		Set<NameRelationship> nameRelations = name.getNameRelations();
		if (nameRelations.size() == 0) {
			return;
		}
		for (NameRelationship nameRelation : nameRelations) {
			String typeLabel = null;
			TaxonNameBase<?, ?> relatedName = null; 

			if (name.equals(nameRelation.getFromName())) {
				typeLabel = labelProvider.getNameRelationTypeLabel( 
								nameRelation.getType());
				relatedName = nameRelation.getToName();
			} else {
				typeLabel = labelProvider.getNameRelationTypeInverseLabel( 
								nameRelation.getType());
				relatedName = nameRelation.getFromName();
			}
			
			setNonEditableInfo(typeLabel + " " + NameHelper.getDisplayName(relatedName));
		}
	}
		
	protected void initTextViewer(TaxonBase<?> taxonBase) {
		
		showNameRelations();
		
		String text = NameHelper.getDisplayNameWithRef(taxonBase);
		
		if (text.length() == 0) {
			initEmptyText();
		} else {
			textViewer.getTextWidget().setText(text);
			textViewer.setCursorToEOL();
		}
		calculateErrors();
		activateParser();
	}
	
	protected void parse(String text) {
		
		// Either composite is not yet fully built, or
		//	the property sheet is writing to it
		if (!isUseParser) {
			return;
		}
		
		// Let others know the parser is active
		isParsing = true;
						
		// Parse the name and paint the text field w any errors
		if (getName() != null) {
			// TODO change to HibernateProxyImpl.deproxy() once it is available
			CdmParserController.parseFullReference(CdmBase.deproxy(getName(), NonViralName.class), text);
		}

		// Any entry of text means the taxon has been changed
		setDirty();
		
		// The parser is no longer active
		isParsing = false;
		
		// Manually refresh the property sheet to reflect changes
		setSelection();
		
		// Show any errors in the name viewer
		calculateErrors();
	}
	
	private void activateParser() {
		isUseParser = true;
	}
	
	// FIXME we do have a naming conflict here 
	protected abstract TaxonNameBase<?, ?> getName();
	
	protected void calculateErrors() {
		textViewer.clearErrors();
		textViewer.setShowParsingError(getName());
	}

	protected void handleSplitText(String text) {
		// Create a synonym in a new homotypic group using text as name
		EditorUtil.executeOperation(new CreateSynonymInNewGroupOperation
				("New Heterotypic Synonym", editor.getUndoContext(), editor.getTaxon(), text, editor));
	}
	
	protected IPropertySource getPropertySourceByName(TaxonNameBase<?, ?> name) {
		if (name == null) {
			return null;
		}
		
		if (name.getClass() == BotanicalName.class) {
			return new BotanicalNamePropertySource((BotanicalName) name);
		}
		if (name.getClass() == ZoologicalName.class) {
			return new ZoologicalNamePropertySource((ZoologicalName) name);
		}
		if (name instanceof NonViralName) {
			return new NonViralNamePropertySource((NonViralName<?>) name);
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.GroupedComposite#refresh()
	 */
	@Override
	public void refresh() {
		super.refresh();
//		initTextViewer((TaxonBase) getData());
		
		// FIXME this is copied from initTextViewer().
		
		showNameRelations();
		
		String text = NameHelper.getDisplayNameWithRef((TaxonBase) getData());
		
		if (text.length() == 0) {
			initEmptyText();
		} else {
			textViewer.getTextWidget().setText(text);
//			textViewer.setCursorToEOL();
		}
//		calculateErrors();
//		activateParser();
	}	
	
	public boolean isDirty(){
		return isDirty;
	}
	
	protected void setDirty(){
		// propagate to editor
		super.setDirty(true);
		// set the dirty state for this composite
		isDirty = true;
	}

	/**
	 * @return the taxonBase
	 */
	public T getTaxonBase() {
		return taxonBase;
	}
}