/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.widgets.FormToolkit;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.CompositeBorderDecorator;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.GroupComposite;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.WidgetTransfer;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.operations.ChangeConceptToSynonymOperation;
import eu.etaxonomy.taxeditor.operations.ChangeHomotypicGroupOperation;

/**
 * @author p.ciardelli
 * @created 15.05.2008
 * @version 1.0
 */
public class TaxonNameEditor extends AbstractTaxonEditor 
		implements INameEditorCompositeRepository {
	private static final Logger logger = Logger.getLogger(TaxonNameEditor.class);
		
	private static final String ID = "eu.etaxonomy.taxeditor.taxonNameEditor";

	public TaxonNameEditor(MultiPageTaxonEditor editor){
		super(editor);
	}
	
	/**
	 * Shared listener that sets dirty state to true 
	 *  when any registered property changes 
	 */
	private PropertyChangeListener taxonChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			if (event.getPropertyName().equals(Resources.PROPERTY_SHEET_CHANGE)) {
				firePropertyChange(PROP_DIRTY);
			}
		}
	};

	private Map<HomotypicalGroup, HomotypicalGroupComposite> homotypicGroupComposites;

	private MisappliedGroupComposite misappliedGroupComposite;

	private ConceptGroupComposite conceptGroupComposite;
	
	@Override
	public void doSave(IProgressMonitor monitor) {}

	@Override
	public void doSaveAs() {}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
				
		super.init(site, input);
		
		// Register listeners for any change in accepted name or set of relations
		taxon.getName().addPropertyChangeListener(taxonChangeListener);
		taxon.addPropertyChangeListener(taxonChangeListener);
	}

	@Override
	public boolean isDirty() {
		return false;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}
	
	@Override
	protected void createManagedForm(Composite composite) {
		
		super.createManagedForm(composite);
		
		Taxon taxon = getTaxon();
		
		firstGroupedComposite = createAcceptedTaxon();
				
		// Draw homotypic group synonyms
		HomotypicalGroup homotypicGroup = taxon.getHomotypicGroup();
		
		if (homotypicGroup != null) {
			List<Synonym> homotypicSynonyms = homotypicGroup.getSynonymsInGroup(taxon.getSec());
			for (Synonym synonym : homotypicSynonyms) {

				// Make sure synonym belongs to the taxon
				if (synonym.getAcceptedTaxa().contains(taxon)) {
					createSynonym(synonym);
				}
			}
		}
		
		// Draw heterotypic synonym groups
		List<HomotypicalGroup> heterotypicGroups = taxon.getHeterotypicSynonymyGroups();
		for (HomotypicalGroup heterotypicGroup : heterotypicGroups) {
			
			// Make sure this is not the taxon's homotypic group
			if (!heterotypicGroup.equals(homotypicGroup)) {
							
				List<Synonym> heterotypicSynonyms = heterotypicGroup.
						getSynonymsInGroup(taxon.getSec());
				for (Synonym synonym : heterotypicSynonyms) {

					// Make sure synonym belongs to the taxon
					if (synonym.getAcceptedTaxa().contains(taxon)) {
						createSynonym(synonym);
					}
				}
			}
		}
		
		// Draw misapplied name elements
		for(Taxon misappliedName : taxon.getMisappliedNames()){
			createMisappliedName(misappliedName);
		}		

		// Draw concept relation elements
		Set<TaxonRelationship> taxonRelations = taxon.getTaxonRelations();
		for (TaxonRelationship relationship : taxonRelations) {
			
			if (relationship.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR()) ||
					relationship.getType().equals(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN())) {
				continue;
			}
							
			createConcept(relationship);
		}		
			
		createDragSupport();
		
		setFocus();

		// Redraw composite
		composite.layout();
	}
		
	
	/**
	 * 
	 */
	private void createDragSupport() {
		// Listen for names being dragged outside of existing homotypic groups -
		//  user wants to create a new group
		Transfer[] types = new Transfer[] {WidgetTransfer.getInstance()};
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
		DropTarget target = new DropTarget(parent, operations);
		target.setTransfer(types);
		target.addDropListener(new DropTargetAdapter() {
			
			public void drop(DropTargetEvent event) {
			
				if (event.data instanceof GroupedComposite) {
					final GroupedComposite composite = (GroupedComposite) event.data;
					if (composite.getDragSource() != null) {
						
							// Execute operations at end of drag event
							composite.getDragSource().addListener(DND.DragEnd, new Listener() {
						
								public void handleEvent(Event event) {
									IUndoableOperation operation = null;
									
									// Synonym being dropped
									if(composite instanceof SynonymComposite){
										Synonym synonym = ((SynonymComposite)composite).getSynonym();
										operation = new ChangeHomotypicGroupOperation
												("change type", getUndoContext(), 
														getTaxon(), synonym, HomotypicalGroup.NewInstance(), TaxonNameEditor.this);
									}
									
									// Misapplied name being dropped
									if(composite instanceof MisappliedNameComposite){
											Taxon misapplication = ((MisappliedNameComposite)composite).getMisappliedName();
											operation = new ChangeConceptToSynonymOperation
													("change misapplied name to synonym", getUndoContext(), 
															getTaxon(), misapplication, HomotypicalGroup.NewInstance(), TaxonNameEditor.this);					
									}
									
									// Execute operation if it's been init'ed
									if (operation == null) {
										logger.warn("User unsuccessfully tried to drop " + composite.getClass());
									} else {
										EditorUtil.executeOperation(operation);
									}
								}
							});							
					}
				}
				

			}		
		});		
	}

	public Composite getAcceptedName() {
		// accepted name should always reside in firstGroupComposite
		if(firstGroupedComposite instanceof AcceptedNameComposite){
			return firstGroupedComposite;
		}
		
//		for(Composite c : getGroupedComposites()){
//			if(c instanceof AcceptedNameComposite){
//				return c;
//			}
//		}
		return null;
	}

	/**
	 * @return
	 */
	private Map<HomotypicalGroup, HomotypicalGroupComposite> getHomotypicGroups() {
		if(homotypicGroupComposites == null){
			homotypicGroupComposites = new HashMap<HomotypicalGroup, HomotypicalGroupComposite>();
		}
		return homotypicGroupComposites;
	}
	
	public HomotypicalGroupComposite getHomotypicGroup(HomotypicalGroup group) {
		return getHomotypicGroups().get(group);
	}

	
	public Composite getMisappliedGroup() {
		return misappliedGroupComposite;
	}

	
	public Composite getMisappliedName(Taxon misappliedName) {
		for(Control child : getMisappliedGroup().getChildren()){
			if(child instanceof MisappliedNameComposite){
				return (MisappliedNameComposite) child;
			}
		}
		return null;
	}

	
	public Composite getSynonym(Synonym synonym) {
				
		GroupComposite groupComposite = getHomotypicGroup(synonym.getHomotypicGroup());
		
		for (Control child : groupComposite.getChildren()){
			if(child instanceof SynonymComposite 
					&& synonym.equals(((SynonymComposite) child).getSynonym())){
				return (SynonymComposite) child;
			}
		}
		
		return null;
	}

	
	public Composite getConceptGroup() {
		return conceptGroupComposite;
	}

	
	public Composite getConcept(Taxon relatedConcept) {
		for (Control child : getConceptGroup().getChildren()){
			if(child instanceof ConceptComposite 
					&& relatedConcept.equals 
					(((ConceptComposite) child).getRelatedTaxon())){
				return (ConceptComposite) child;
			}
		}

		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor#getID()
	 */
	@Override
	public String getID() {
		return TaxonNameEditor.ID;
	}
	
	/**
	 * @return a Set containing all composites that have been edited
	 */
	public Set<NameComposite<TaxonBase>> getDirtyNames(){
		Set<NameComposite<TaxonBase>> dirtyNames = new HashSet<NameComposite<TaxonBase>>();
		
		for(GroupedComposite composite : getGroupedComposites()){
			if(composite instanceof NameComposite && ((NameComposite)composite).isDirty()){
				dirtyNames.add((NameComposite) composite);
			}
		}
		
		return dirtyNames;
	}
	
	/***********************************************************************/
	
	private GroupedComposite createAcceptedTaxon() {
		// Create a homotypic group composite for the accepted taxon
		HomotypicalGroup group = getTaxon().getHomotypicGroup();
		Assert.isNotNull(group, "Taxon does not have a homotypic group");
		
		createHomotypicalGroup(group);
		
		// Create a name composite for the accepted taxon
		return new AcceptedNameComposite(this, getHomotypicGroup(group));
	}
	
	private HomotypicalGroupComposite createHomotypicalGroup(HomotypicalGroup group) {

		// Create the group composite
		HomotypicalGroupComposite groupComposite = new HomotypicalGroupComposite(this, getTopLevelComposite(), group);

		
		//groupComposite.addFocusListener(new CompositeBorderDecorator(groupComposite, editor.getManagedForm()));		
				
		groupComposite.setData(FormToolkit.KEY_DRAW_BORDER, FormToolkit.TEXT_BORDER);
		getManagedForm().getToolkit().paintBordersFor(groupComposite.getParent());
		
		getHomotypicGroups().put(group, groupComposite);
		
		return groupComposite;
	}


	private Composite createSynonym(Synonym synonym) {		
		// Get the synonym's homotypic group
		HomotypicalGroup group = synonym.getHomotypicGroup();
		Assert.isNotNull(group, "Synonym does not have a homotypic group");
		
		// If the group doesn't yet have a composite, create one and add it to the repository
		HomotypicalGroupComposite groupComposite = getHomotypicGroup(group);
		if (groupComposite == null) {
			groupComposite = createHomotypicalGroup(group);
		}
				
		// Create a synonym composite in the homotypical group
		Composite synonymComposite = new SynonymComposite(this, groupComposite, synonym);
				
		return synonymComposite;
	}
	
	public Composite createMisappliedName(Taxon misappliedName) {
		
		// If there is no composite for misapplied names, 
		//	create one and add it to the repository
		Composite groupComposite = getMisappliedGroup();
		if (groupComposite == null) {
			groupComposite = createMisappliedGroup();
		}
		
		// Create the name's composite
		Composite composite = new MisappliedNameComposite(this, groupComposite, misappliedName);
		
		return composite;
	}
	
	
	public MisappliedGroupComposite createMisappliedGroup() {
		// Create the group composite
		misappliedGroupComposite = new MisappliedGroupComposite(this, this.getTopLevelComposite());
		misappliedGroupComposite.addFocusListener(new CompositeBorderDecorator(misappliedGroupComposite, this.getManagedForm()));
		
		// Put the group composite before concept group composite, if any
		Composite conceptGroupComposite = this.getConceptGroup();
		if (conceptGroupComposite != null) {
			misappliedGroupComposite.moveAbove(conceptGroupComposite);
		}
		
		return misappliedGroupComposite;
	}
	
	
	public Composite createConcept(TaxonRelationship relationship) {	
		// If there is no composite for misapplied names, 
		//	create one and add it to the repository
		Composite groupComposite = this.getConceptGroup();
		if (groupComposite == null) {
			groupComposite = createConceptGroup();
		}
		
		// Create the name's composite
		Composite composite = ConceptComposite.getNewInstance(this, groupComposite, relationship);
				
		return composite;
	}

	
	public Composite createConceptGroup() {
		// Create the group composite
		conceptGroupComposite = new ConceptGroupComposite(this, this.getTopLevelComposite());
		conceptGroupComposite.addFocusListener(new CompositeBorderDecorator(conceptGroupComposite, this.getManagedForm()));
		
		// Put the group composite after misapplied group composite, if any
		Composite misappliedGroupComposite = this.getMisappliedGroup();
		if (misappliedGroupComposite != null) {
			conceptGroupComposite.moveBelow(misappliedGroupComposite);
		}
		
		return conceptGroupComposite;

	}
}