/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;

/**
 * @author p.ciardelli
 * @created 03.11.2008
 * @version 1.0
 */
public class ScientificNamePropertySource implements ICdmBasePropertySource {
	private static final Logger logger = Logger
			.getLogger(ScientificNamePropertySource.class);

	NonViralName name;
	
    // Property unique keys
	public static final String P_ID_SEARCH = "P_ID_SEARCH";
	public static final String P_ID_EDITABLECACHE = "P_ID_EDITABLECACHE";	
	public static final String P_ID_UNINOMIAL = "P_ID_UNINOMIAL";
	public static final String P_ID_INFRAGENERICEP = "P_ID_INFRAGENERICEP";
	public static final String P_ID_INFRASPECIFICEP = "P_ID_INFRASPECIFICEP";
	public static final String P_ID_SPECIESEP = "P_ID_SPECIESEP";
	public static final String P_ID_PROTECT_CACHE = "P_ID_PROTECT_CACHE";
	public static final String P_ID_APPENDEDPHRASE = "P_ID_APPENDEDPHRASE";
	public static final String P_ID_AUTHORSHIPCACHE = "P_ID_AUTHORSHIPCACHE";
		
    // Property display keys
	public static final String P_SEARCH = "Search";
	public static final String P_EDITABLECACHE = "Editable Cache";
	public static final String P_PROTECT_CACHE = "Protect Cache from overwriting?";
	public static final String P_UNINOMIAL = "Uninomial";
	public static final String P_INFRAGENERICEP = "Infrageneric Epithet";
	public static final String P_INFRASPECIFICEP = "Infraspecific Epithet";
	public static final String P_SPECIESEP = "Specific Epithet";
	public static final String P_AUTHORSHIPCACHE = "Authorship";
	public static final String P_APPENDEDPHRASE = "Appended Phrase";	
	
	private static final int CACHE_NOT_PROTECTED = 0;
	private static final int CACHE_PROTECTED = 1;	
			
	public ScientificNamePropertySource(NonViralName name) {
		
		// Default type of ReferenceBase is Generic
		// FIXME disabled
//		if (name == null) {
//			name = PreferencesUtil.getInstanceOfPreferredNameClass();
//		}		
		this.name = name;

		initDescriptors();
	}

	protected void initDescriptors() {
		
		List<String> displayFields = new ArrayList<String>();
			
		// Cache fields
//		displayFields.add(P_ID_SEARCH);
		displayFields.add(P_ID_EDITABLECACHE);
		displayFields.add(P_ID_PROTECT_CACHE);
		
		// Uninomial
    	displayFields.add(P_ID_UNINOMIAL);
    	
    	if (name.isSupraGeneric() || name.isGenus()) { // Rank is higher than GENUS or equals GENUS
    	}
    	else if (name.isInfraGeneric()) { // lower than GENUS and higher than SPECIES
    		displayFields.add(P_ID_INFRAGENERICEP);
    	}   
    	else if (name.isSpecies()) { // Rank equals SPECIES
    		displayFields.add(P_ID_SPECIESEP);
    	}
    	else if (name.isInfraSpecific()) { // Rank is lower than SPECIES
    		displayFields.add(P_ID_SPECIESEP);
    		displayFields.add(P_ID_INFRASPECIFICEP);
    	} else {
    		displayFields.add(P_ID_SPECIESEP);
    	}
    	displayFields.add(P_ID_AUTHORSHIPCACHE);
    	displayFields.add(P_ID_APPENDEDPHRASE);
    
		
		for (String field : displayFields) {
			addDescriptor(field);
		}
	}

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	protected void addDescriptor(String id) {
		
		// Reference search
		if (id.equals(P_ID_SEARCH)) {
//			descriptors.addElement(
//					new ReferenceSearchDescriptor(P_ID_SEARCH, P_SEARCH, getSearchType()) {
//						protected void saveReference(ReferenceBase reference) {
//							setPropertyValue(P_ID_SEARCH, reference);
//						}
//			});
		}
		
		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_EDITABLECACHE, P_EDITABLECACHE));
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			descriptors.addElement(
					new ComboBoxPropertyDescriptor(P_ID_PROTECT_CACHE, P_PROTECT_CACHE, 
							new String[] {"no", "yes"}));
		}
		
		boolean isProtectedCache = name.isProtectedTitleCache();
//		boolean isProtectedCache = false;
		
		// Uninomial (aka Genus)
		if (id.equals(P_ID_UNINOMIAL)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_UNINOMIAL, P_UNINOMIAL) :
							new TextPropertyDescriptor(P_ID_UNINOMIAL, P_UNINOMIAL));
		}
		
		// Infrageneric epithet
		if (id.equals(P_ID_INFRAGENERICEP)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_INFRAGENERICEP, P_INFRAGENERICEP) :
							new TextPropertyDescriptor(P_ID_INFRAGENERICEP, P_INFRAGENERICEP));
		}
		
		// Specific epithet
		if (id.equals(P_ID_SPECIESEP)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_SPECIESEP, P_SPECIESEP) :
							new TextPropertyDescriptor(P_ID_SPECIESEP, P_SPECIESEP));
		}
		
		// Infraspecific epithet
		if (id.equals(P_ID_INFRASPECIFICEP)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_INFRASPECIFICEP, P_INFRASPECIFICEP) :
							new TextPropertyDescriptor(P_ID_INFRASPECIFICEP, P_INFRASPECIFICEP));
		}
		
		// Appended phrase
		if (id.equals(P_ID_APPENDEDPHRASE)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_APPENDEDPHRASE, P_APPENDEDPHRASE) :
							new TextPropertyDescriptor(P_ID_APPENDEDPHRASE, P_APPENDEDPHRASE));
		}
		
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_AUTHORSHIPCACHE, P_AUTHORSHIPCACHE));
		}	
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return CdmUtils.Nz(name.getTitleCache());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
				
		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			return (CdmUtils.Nz(name.getTitleCache()));
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			if (name.isProtectedTitleCache()) {
				return CACHE_PROTECTED;
			} else {
				return CACHE_NOT_PROTECTED;
			}
		}
		
		// Uninomial (aka Genus)        
		if (id.equals(P_ID_UNINOMIAL)) {
			return CdmUtils.Nz(name.getGenusOrUninomial());
		}
	
		// Infrageneric epithet		
        if (id.equals(P_ID_INFRAGENERICEP)) {
			return CdmUtils.Nz(name.getInfraGenericEpithet());
        }
		
		// Specific epithet        
        if (id.equals(P_ID_SPECIESEP)) {
			return CdmUtils.Nz(name.getSpecificEpithet());
        }
        
		// Infraspecific epithet        
        if (id.equals(P_ID_INFRASPECIFICEP)) {
			return CdmUtils.Nz(name.getInfraSpecificEpithet());
        }
        

		// Appended phrase        
        if (id.equals(P_ID_APPENDEDPHRASE)) {
        	return CdmUtils.Nz(name.getAppendedPhrase());
        }
        
		// Authorship cache        
        if (id.equals(P_ID_AUTHORSHIPCACHE)) {
        	if (name.isProtectedTitleCache()) {
        		return CdmUtils.Nz(name.getAuthorshipCache());
        	} else {
        		return new NomenclaturalAuthorshipPropertySource(name);
        	}
        }
								
		return "";
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		
		// Search result
		if (id.equals(P_ID_SEARCH)) {
			if (value instanceof NonViralName) {
				name = (NonViralName) value;
			}
		}

		// Protect cache? 
		if (id.equals(P_ID_PROTECT_CACHE)) {
			logger.warn(((Integer) value).intValue() == CACHE_PROTECTED);
			name.setProtectedTitleCache(
					((Integer) value).intValue() == CACHE_PROTECTED);
		}
				
		// Uninomial (aka Genus)		
		if (id.equals(P_ID_UNINOMIAL)) {
			name.setGenusOrUninomial((String) value);
		}
		
		// Infrageneric epithet		
        if (id.equals(P_ID_INFRAGENERICEP)) {
			name.setInfraGenericEpithet((String) value);
        }
        
        // Specific epithet        
        if (id.equals(P_ID_SPECIESEP)) {
			name.setSpecificEpithet((String) value);
        }
        
		// Infraspecific epithet        
        if (id.equals(P_ID_INFRASPECIFICEP)) {
			name.setInfraSpecificEpithet((String) value);
        }
        
		// Appended phrase        
        if (id.equals(P_ID_APPENDEDPHRASE)) {
			name.setAppendedPhrase((String) value);
        }
        
		// Authorship cache        
//        if (id.equals(P_ID_AUTHORSHIPCACHE)) {
////        	name.setProtectedAuthorshipCache(true);
//			name.setAuthorshipCache((String) value);
//        }
        
		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			name.setTitleCache((String) value);
//		} else {
//			name.setTitleCache(name.generateTitle(), name.isProtectedTitleCache());
		}
		
		propertyChangeSupport.firePropertyChange(Resources.PROPERTY_SHEET_CHANGE, null, name);
	}
	

	public String toString() {
		return CdmUtils.Nz(name.getTitleCache());
	}

	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(
			PropertyChangeListener listener) {
		propertyChangeSupport.addPropertyChangeListener(listener);
	}
}