/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Set;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.IReferenceSearch;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferenceSearchDescriptor;
import eu.etaxonomy.taxeditor.propertysheet.type.TypeCollectionPropertySource;
import eu.etaxonomy.taxeditor.propertysheet.type.TypePropertyDescriptor;

/**
 * @author p.ciardelli
 * @created 11.11.2008
 * @version 1.0
 */
public class TaxonBasePropertySource implements ICdmBasePropertySource {
	private static final Logger logger = Logger
			.getLogger(TaxonBasePropertySource.class);
	
	private TaxonBase<?> taxonBase;

    // Property unique keys
	public static final String P_ID_TAXONNAME = "P_ID_TAXONNAME";
	public static final String P_ID_TAXONSEC = "P_ID_TAXONSEC";
	public static final String P_ID_TYPES = "P_ID_TYPES";
	
    // Property display keys
	public String P_TAXONNAME;
	public static final String P_TAXONSEC = "Secundum";
	public String P_TYPES = "Name Types";	
	
	public TaxonBasePropertySource(TaxonBase<?> taxon, String nameTitle) {
		this.taxonBase = taxon;
		
		this.P_TAXONNAME = nameTitle;
		
		if (taxon != null && taxon.getName() != null && 
				!NameHelper.isNameSupraSpecific(taxon.getName())) {
			P_TYPES = "Specimen Types";
		}
		
		addDescriptor(P_ID_TAXONNAME);
		addDescriptor(P_ID_TAXONSEC);
		addDescriptor(P_ID_TYPES);		
	}

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	protected void addDescriptor(String id) {
		if (id.equals(P_ID_TAXONNAME)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_TAXONNAME, P_TAXONNAME));
		}		
		if (id.equals(P_ID_TAXONSEC)) {
			descriptors.addElement(new ReferenceSearchDescriptor(P_ID_TAXONSEC, P_TAXONSEC, IReferenceSearch.BIBREF) {
				protected void saveReference(ReferenceBase reference) {
					setPropertyValue(P_ID_TAXONSEC, reference);
				}
			});
		}
		if (id.equals(P_ID_TYPES)) {
			descriptors.addElement(
				new TypePropertyDescriptor(P_ID_TYPES, P_TYPES, taxonBase.getName()) {
					protected void saveTypes(Set set) {
						setPropertyValue(P_ID_TYPES, set);
					}
				}
			);
		};
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {

		// Edit taxon's name
		if (id.equals(P_ID_TAXONNAME)) {
			
			if (taxonBase == null) {
				logger.warn("no taxon");
				return null;
			}
			TaxonNameBase<?, ?> name = taxonBase.getName();
			
			// Create taxon name as necessary
			if (name == null) {
				name = NonViralName.NewInstance(Rank.SPECIES());
			}
			name = CdmBase.deproxy(name, TaxonNameBase.class);
			// Send taxon name to appropriate property source for submenu
			if (name instanceof BotanicalName) {
				return new BotanicalNamePropertySource((BotanicalName) name);
			}
			if (name instanceof ZoologicalName) {
				return new ZoologicalNamePropertySource((ZoologicalName) name);
			}
			if (name instanceof NonViralName) {
				return new NonViralNamePropertySource((NonViralName<?>) name);
			}
			
		}		
		
		// Edit taxon's sec. reference
		if (id.equals(P_ID_TAXONSEC)) {

			if (taxonBase == null) {
				return null;
			}
			ReferenceBase<?> sec = taxonBase.getSec();
			
			// Create property source for submenu
			ReferencePropertySource secPropertySource = new ReferencePropertySource(sec);
			
			// Add listener to notify taxon of all changes to sec
			secPropertySource.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (evt.getNewValue() instanceof ReferenceBase) {	
						taxonBase.setSec((ReferenceBase<?>) evt.getNewValue());
					}
				}
			});
			return secPropertySource;
		}
		
		if (id.equals(P_ID_TYPES)) {
			if (taxonBase.getName() != null) {
				// TODO return NameTypeDesignations
				TaxonNameBase<?, ?> name = taxonBase.getName();
				Set<?> typeDesignations = null;
				if (NameHelper.isNameSupraSpecific(name)) {
					typeDesignations = name.getNameTypeDesignations();
				} else {
					typeDesignations = name.getSpecimenTypeDesignations();
				}
				return new TypeCollectionPropertySource(name, typeDesignations);
			}
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
        if (id.equals(P_ID_TAXONSEC)) {
        	if (value instanceof ReferenceBase) {
        		taxonBase.setSec((ReferenceBase<?>) value);
        	}
        }
	}

	/**
	 * @return the taxonBase
	 */
	public TaxonBase<?> getTaxonBase() {
		return taxonBase;
	}
}
