/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Status;
import org.springframework.security.providers.ProviderManager;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.impl.TaxonServiceConfiguratorImpl;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.Article;
import eu.etaxonomy.cdm.model.reference.Book;
import eu.etaxonomy.cdm.model.reference.BookSection;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.datasource.DatasourceLauncherDialog;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * This implementation of ICdmDataRepository depends on hibernate sessions to store the data correctly 
 * for the current session. No state is held in this class.
 * 
 * Only methods that either get or manipulate data are exposed here. So this class acts as a facade 
 * for the methods in cdmlib-service. 
 * 
 * 
 * @author n.hoffmann
 * @created 17.03.2009
 * @version 1.0
 */
public class CdmStore{
	private static final Logger logger = Logger.getLogger(CdmStore.class);
	
	// FIXME change this to ClassPathResources as soon as it is included into the plugin
	private static String DEFAULT_APPLICATION_CONTEXT = null;
	private static DbSchemaValidation DEFAULT_DB_SCHEMA_VALIDATION = DbSchemaValidation.UPDATE;
	
	private static CdmStore instance;
	
	private CdmApplicationController applicationController;
	
	private ConversationHolder globalReadOnlyConversation;
	
	private static DbSchemaValidation dbSchemaValidation;

	private static LoginManager loginManager;
	
	/**
	 * 
	 * @return
	 */
	public static CdmStore getDefault(){
		return getDefault(DEFAULT_APPLICATION_CONTEXT);
	}
	
	/**
	 * 
	 * @param applicationContextBean
	 * @return
	 */
	public static CdmStore getDefault(String applicationContextBean){
		if(instance == null){
			logger.info("Initializing application context ...");
			
			// Prompt user for datasource if there are more than one
			if(! PreferencesUtil.getPreferenceStore().getBoolean(PreferencesUtil.HIDE_DATASOURCE_CHOOSER) 
				&& CdmDataSourceRepository.getDefault().getAll().size() > 1){
				DatasourceLauncherDialog chooseDataSource = new DatasourceLauncherDialog(StoreUtil.getShell());
				if(chooseDataSource.open() == Status.OK){
					instance = initialize(applicationContextBean);

				}else{
					// no datasource chosen.
					logger.warn("No datasource chosen. Exiting now.");
					System.exit(1);
				}	
			}else{
				// Start default cdm database
				instance = initialize(applicationContextBean);
			}
		
		}
		return instance;
	}
	
	private static CdmStore initialize(String applicationContextBean){	
		
		ICdmDataSource cdmDatasource = CdmDataSourceRepository.getDefault().getCurrentDataSource();
		
		CdmStore instance = new CdmStore(cdmDatasource, getDbSchemaValidation(), applicationContextBean);
		
		logger.info("Application context initialized.");
		
		return instance;
	}

	/**
	 * @return
	 */
	private static DbSchemaValidation getDbSchemaValidation() {
		return (dbSchemaValidation == null) ? DEFAULT_DB_SCHEMA_VALIDATION : dbSchemaValidation;
	}

	/**
	 * 
	 */
	private CdmStore(ICdmDataSource dataSource, DbSchemaValidation dbSchemaValidation, String applicationContextBean) {
		// TODO application context bean is not honored by application controller at the moment. 
		// cdmDefaultApplicationController bean gets loaded per default always.
		
		try {
			CdmStore.DEFAULT_APPLICATION_CONTEXT = applicationContextBean;
//			applicationController = CdmApplicationController.NewInstance(applicationContextBean, dataSource, dbSchemaValidation, false);
			
//			logger.warn("OMITTING TERM LOADING FOR DEBUGGING");
//			applicationController = CdmApplicationController.NewInstance(dataSource, dbSchemaValidation, true);
			applicationController = CdmApplicationController.NewInstance(dataSource, dbSchemaValidation);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}


	public ReferenceBase<?> getDefaultSec() {
		// TODO why is this returning null? and of course, replace w the real deal
		return applicationController.getReferenceService().getReferenceByUuid(
				UUID.fromString("f3593c18-a8d2-4e51-bdad-0befbf8fb2d1"));
	}
	
	/**
	 * @deprecated use taxonomic tree instead
	 */
	public List<Taxon> getRootTaxa() {
		boolean onlyWithChildren = false;
		boolean withMisapplications = true;
		
		return applicationController.getTaxonService().getRootTaxa(
				getDefaultSec(), onlyWithChildren, withMisapplications);
	}
	
	/**
	 * @return
	 */
	public List<TaxonomicTree> getTaxonomicTrees() {
		return applicationController.getTaxonService().listTaxonomicTrees(null, null, null, null);
	}
		
	
	private CdmApplicationController getApplicationControllerInternal(){
		return applicationController;
	}
	
	public static CdmApplicationController getApplicationController(){
		return getDefault().getApplicationControllerInternal();
	}

	
	/**
	 * Create a new conversation and bind resources to it
	 * 
	 * @return
	 */
	public static ConversationHolder NewConversation(){
		
		CdmStore store = getDefault();
		CdmApplicationController controller = store.getApplicationController();
		
		ConversationHolder conversation = controller.NewConversation();

		return conversation;
	}

	/**
	 * Creates a new conversation, binds resources to the conversation and 
	 * start a transaction for this conversation.
	 * 
	 * @return
	 */
	public static ConversationHolder NewTransactionalConversation() {
		ConversationHolder conversation = NewConversation();
		
		conversation.startTransaction();
		return conversation;
	}
	
	
	private ConversationHolder getGlobalReadOnlyConversation() {
		ConversationHolder conversation = globalReadOnlyConversation == null 
				? NewConversation() 
				: globalReadOnlyConversation;
		conversation.bind();
		return conversation;
	}
	
	public static ConversationHolder getGlobalConversation(){
		return getDefault().getGlobalReadOnlyConversation();
	}
	

	/*
	 * EXPOSING SERVICES
	 */
	
	public static ITaxonService getTaxonService(){ return getDefault().getApplicationControllerInternal().getTaxonService();}
	
	public static INameService getNameService(){ return getDefault().getApplicationControllerInternal().getNameService();}
	
	public static IReferenceService getReferenceService(){ return getDefault().getApplicationControllerInternal().getReferenceService();}
	
	public static ILocationService getLocationService(){ return getDefault().getApplicationControllerInternal().getLocationService();}

	public static ProviderManager getAuthenticationManager() { return getDefault().getApplicationControllerInternal().getAuthenticationManager();}

	public static IUserService getUserService() { return getDefault().getApplicationControllerInternal().getUserService(); }
	
	/**
	 * @param searchText
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static List<TaxonNameBase> searchNameString(String searchText) {
		List<TaxonNameBase> resultSet = new ArrayList<TaxonNameBase>();
		resultSet.addAll(getNameService()
				.getNamesByName(searchText.replace("*", "%")));
		return resultSet;
	}
	
	@Deprecated
	private List<IdentifiableEntity> findTaxaByName(String queryString, boolean restrictToTaxonObjs) {

		ITaxonServiceConfigurator configurator = new TaxonServiceConfiguratorImpl();
		
		configurator.setSearchString(queryString.trim());
		configurator.setDoTaxa(true);
		configurator.setMatchMode(MatchMode.BEGINNING);
		if (restrictToTaxonObjs) {
			configurator.setDoNamesWithoutTaxa(false);
			configurator.setDoSynonyms(false);			
		} else {
			configurator.setDoNamesWithoutTaxa(true);
			configurator.setDoSynonyms(true);
		}
		configurator.setSec(null);
		configurator.setPageNumber(0);
		// TODO currently limit results to 1000
		configurator.setPageSize(1000);
		
		List<IdentifiableEntity> result = getTaxonService().findTaxaAndNames(configurator).getRecords();
		
		return result;
	}
	
	
	
	private List<IdentifiableEntity> findTaxaByName(ITaxonServiceConfigurator configurator){
		
		List<IdentifiableEntity> result = getTaxonService().findTaxaAndNames(configurator).getRecords();
		
		return result;
	}
	
	public static List<IdentifiableEntity> findNames(ITaxonServiceConfigurator configurator){
		return getDefault().findTaxaByName(configurator);
	}
	
	
	
	public static List<IdentifiableEntity> searchTaxaByName(String queryString){
		return getDefault().findTaxaByName(queryString, false);
	}

	public static List<IdentifiableEntity> searchTaxaByName(String queryString, boolean restrictToTaxonObjs){
		return getDefault().findTaxaByName(queryString, restrictToTaxonObjs);
	}
	
	/**
	 * Searches for references by string. "%" is used as a wildcard.
	 * 
	 * @param text
	 * @return 
	 */
	public static List<ReferenceBase> getReferencesByTitle(String reference) {
		
		reference = reference.replace("*", "%");
		List<ReferenceBase> resultsList = null;
		try {
			resultsList = getReferenceService().getReferencesByTitle(reference);
		} catch (RuntimeException e) {
//			MessageDialog.openError(GlobalController.getShell(), "Search reference error", 
//					"Reference search returned an error. This could be a Hibernate concurrency problem. " +
//					"Please try saving your work, then searching again.");
			e.printStackTrace();
		}
		return resultsList;
	}
	
	/**
	 * FIXME mock
	 * 
	 * @return
	 */
	public static Language getDefaultLanguage(){
		return Language.ENGLISH();
	}

	/**
	 * @return
	 */
	public static LinkedHashMap<Class<?>, String> getReferenceTypes() {
		LinkedHashMap<Class<?>, String> nomReferenceTypeMap = new LinkedHashMap<Class<?>, String>();

//		referenceTypeMap.put(BibtexReference.class, "BibTeX Reference");
		nomReferenceTypeMap.put(Article.class, "Article");
		nomReferenceTypeMap.put(Generic.class, "Generic");
		nomReferenceTypeMap.put(Book.class, "Book");
		nomReferenceTypeMap.put(BookSection.class, "Book Section");
		
		return nomReferenceTypeMap;
	}
	
	/**
	 * incomplete! do note use!
	 */
	public static LoginManager getLoginManager(){
		if(loginManager == null){
			loginManager = new LoginManager();
		}
		return loginManager;
	}
}
